;-----------------------------------------------------------------------------;
; Modified Multiple Boot Manager with ct6b by GarakutaEn 2021.4
;-----------------------------------------------------------------------------;
; Modified R0.39  Final Version. Changed license to GNU GPL.
; OPTASM 1.6 or later is required to build this program.
;-----------------------------------------------------------------------------;
;
; MBM R0.39  Copyright (C)2008  ChaN
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;-----------------------------------------------------------------------------;

page	60, 130
include	stddef.inc
include	str.inc
include	file.inc
include	ibmpc.inc


	VER	equ	'0.39 '	; 5bytes
	N_MBM	equ	18	; Number of MBM sectors (see end_of_mbm)

purge	PRINT
PRINT	macro	str
	IFNB <str>
	mov	si, ofs str
	ENDIF
	 call	disp_str
	endm



PartitionTbl	struc
 BootID		db	?	; u[gW 80h:Active, 00:Inactive
 StartHead	db	?	; Jnwbhԍ
 StartSectCyl	dw	?	; JnZN^ԍƃV_ԍ
 SystemID	db	?	; OSʃR[h
 EndHead	db	?	; ŏIwbhԍ
 EndSectCyl	dw	?	; ŏIZN^ԍ
 PartitionOfs	dw	?,?	; JnZN^ (LBA)
 PartitionSize	dw	?,?	; TCY (ZN^)
PartitionTbl	ends

NameItem	struc
 PartitionName	db	30 dup(?)
NameItem	ends

Parms		struc
 MpMbmSect	db	?	; MBM{̊JnZN^
 MpFlags	db	?	; b0:ڍו\, b1:Help\, b2:DefaultŒ
 MpMbmSum	dw	?	; MBM{̃T
 MpDef		dw	?	; Default(0..16)
 MpTime		dw	?	; N^C}[s]
 MpBoot		dw	?,?	; Boot mode
 MpMask		dw	?,?	; Mask mode
 MpExp		dw	?	; Pre-expand
 MpCyl		dw	?,?	; gj[p
 MpSects	dw	?	; 
 MpSum		dw	?	; /
Parms		ends

				;EditFlags bit definition
F_NA		equ	BIT3	; Not aligned allocation
F_ER		equ	BIT2	; Allocation error
F_CC		equ	BIT1	; Cylinder changed
F_IC		equ	BIT0	; ID changed

.model t
.186
.code
		DEF_PSP

;-----------------------------------------------;
; DOSR}hC

start:		PRINT	StrDosOpen
		GET_CMD_PAR Argc,Argv
		mov	Drive, 80h
	_skip e, Argc, 0			; R}hʕ
		mov	si, ofs CmdList
		 call	chk_cmd
		je	install
		 call	chk_cmd
		je	install_high
		 call	chk_cmd
		je	do_edit
		 call	chk_cmd
		je	clear_menu
		 call	chk_cmd
		je	set_menu
		 call	chk_cmd
		je	create_fd
		 call	chk_cmd
		je	create_bin
	_endif
usage:
		PRINT	StrDosUsage
		jmp	exit

chk_cmd:
		mov	bx, Argv[0]
	_while
		mov	ah,[bx]
		inc	bx
		CAPS	ah
		lodsb
	 _if ne, al, ah
	  _while ne, al, 0
		lodsb
	  _rept
		inc	al
		ret
	 _endif
	_rept ne, al, 0
		ret

CmdList		db	'INSTALL',0
		db	'INSTALL_H',0
		db	'EDIT',0
		db	'CLRMENU',0
		db	'SETMENU',0
		db	'FDIMAGE',0
		db	'MBMIMAGE',0


;-----------------------------------------------;
; e[uGfB^(DOS[h)

do_edit:
		$VIDEO  0fh		;Save original video mode
		push	ax		;/
		$DOS 63h, 0		;Save current code set
		@32 <clr ax>		;
		@32 <xchg ax,[si]>	;
		@32 <push ax>		;
		push	si,ds		;
		movseg	ds,cs		;/
		$VIDEO  0, 3		;Select Video mode #3
		mov	bl, 0		;Disable brink
		$VIDEO  10h, 3		;/
		 call	editor		;Execute editor
		pop	ds,si		;Restore current code set
		@32 <pop [si]>		;
		movseg	ds,cs		;/
		pop	ax		;Restore original video mode
		$VIDEO 0		;/
		jmp	exit


;-----------------------------------------------;
; gj[R[h폜(DOS[h)

clear_menu:
		 call	read_mbr0		; Load MBR
		jc	io_err			; 
		 call	chk_inst		; Check installation
		mov	MbrParms.MpSects, 0	; Clear Menu inidicater
		 call	write_mbr0		; Update MBR
		jc	io_err			;
		PRINT  MesOK
		jmp	exit


;-----------------------------------------------;
; gj[R[h(DOS[h)

set_menu:	call	read_mbr_parm	;Load MBR and drive parms
		jc	io_err		;/
	_if e, Lba, 0			;Check LBA
		PRINT	MesNoLba	;
		jmp	exit		;
	_endif				;/
		 call	chk_inst	;Check installed

	_while				; t@C
		PRINT  ReqFile
		mov	bx, ofs Pool
		 call	get_str
		OPEN_FILE 0, <ofs Pool>
	 _break nc
		PRINT	NoFile
	_rept
		mov	Hand, ax

	_while				; V_ԍ
		PRINT  ReqCyl
		@32 <mov ax, Cylinders>
		dec	ax
		clr	dl
		 call	disp_num4
		PRINT
		mov	di, ofs Pool
		mov	bptr[di], 0
		@32 <mov ax, MbrParms.MpCyl>
		@32 <cmp ax, -1>,-1
	 @ifn e,<call	make_num4>
		mov	bx, ofs Pool
		 call	get_str_cont
		 call	get_value
	_rept c
	_if ae, <@32 <cmp cx, Cylinders>>
		PRINT  MesInvalid
		jmp	exit
	_endif
		@32 <mov MbrParms.MpCyl, cx>

		clr	bp, bx		; V_gp`FbN
	_repeat 4
		push	cx
	 _skip e, MbrPartition[bp].SystemID, 0
		@32 <mov ax,MbrPartition[bp].PartitionOfs>
		 call	cyl_num
	  _if be, <@32 <cmp ax, MbrParms.MpCyl>>
		@32 <mov ax,MbrPartition[bp].PartitionOfs>
		@32 <add ax,MbrPartition[bp].PartitionSize>
		 call	cyl_num
	   _if a, <@32 <cmp ax, MbrParms.MpCyl>>
		set	bh
	 ___endif
		pop	cx
		add	bp,type PartitionTbl
	_loop
	_if ne, bh, 0			; gpȂ^ӂ̊mF
		PRINT  MesCyl
		PRINT  MesCont
		mov	bx, ofs Pool
		 call	get_str
		mov	al, Pool
	 _skip e, al,'Y'
		jmp	exit
	__endif

		PRINT  MesWrite
		clr	cx,dx
		SEEK_FP Hand, 2
		push	dx,ax
		clr	cx,dx
		SEEK_FP Hand, 0
		@32 <pop ax>
		@32 <add ax,511>,0
		@32 <shr ax,9>
		jz	io_err
		cmp	ax, 1200		;Menu size ?
		jae	too_large		;/
		mov	MbrParms.MpSects, ax
		@32 <mov cx, ax>
		 call	cyl_size
		@32 <mov dx, MbrParms.MpCyl>
		@32 <inc dx>
		@32 <mul dx>
		@32 <mov di,ax>
		@32 <sub di,cx>
		mov	MbrParms.MpSum, 0
		mov	bx, -1
	_repeat
		push	cx
	 _if ae, bx, <ofs Pool[512*32]>
		READ_FILE Hand, <ofs Pool>, 512*32
		jc	io_err
		mov	bx, dx
	 _endif
		mov	si, bx
	 _repeat 512/2
		lodsw
		add	MbrParms.MpSum, ax
	 _loop
		@32 <push 0>
		@32 <push di>
		push	ds,bx, 1, 16
		mov	si, sp
		mov	dl, 80h
		$DISK  43h,0
		lea	sp, [si].16
		jc	io_err
		pop	cx
		add	bx, 512
		@32 <inc di>
	_loop
		 call	write_mbr	; Update
		jc	io_err
		PRINT  MesOK
		jmp	exit

too_large:
		PRINT  MesLarge
		jmp	exit


chk_inst:	; u[g}l[W̃CXg[`FbN
		CMP_STR <ofs MbrBuffer[2]>,<ofs MbmSign>,<len MbmSign>
	_skip e
		PRINT  MesNoInst
		jmp	exit
	_endif
		ret



disp_num4:	clr	bx		; l̕\
	_while				; Call:
		inc	bl		;  eax = number, dl = number of digits
		mov	cl, 10
		 call	div_eax1
		add	cl, '0'
		push	cx
		@32 <and ax, ax>
	_rept nz
	_while b, bl,dl
		inc	bx
		push	' '
	_rept
	_repeat bx
		pop	ax
		 call	disp_chr
	_loop
		ret


make_num4:	clr	bx		; l̍쐬
	_while				; Call:
		inc	bx		;  eax = number
		mov	cl, 10
		 call	div_eax1
		add	cl, '0'
		push	cx
		@32 <and ax, ax>
	_rept nz
	_repeat bx
		pop	ax
		stosb
	_loop
		mov	bptr[di],0
		ret


;-----------------------------------------------;
; FDC[W쐬 (DOS[h)

create_fd:
		MAKE_FILE 0, <ofs ImgFile>
		jc	exit
		mov	Hand, ax
		FIL_STR <ofs Pool>, 1024, 0
		MOV_MEM <ofs fdbs_code>, <ofs Pool>, <ofs fdbs_end-fdbs_code>
		mov	wptr Pool[510], 0aa55h
		WRITE_FILE Hand, <ofs Pool>, 512	;First 36 sectors (Cy:0)
		WRITE_FILE <>, 100h, 512*35		;/
	_repeat 18*2*80-36				;Following sectors (Cy:1..79)
		push	cx				;
		WRITE_FILE <>, <ofs Pool[512]>, 512	;
		pop	cx				;
	_loop						;/
		CLOSE_FILE <>
		PRINT  MesOK
		jmp	exit

ImgFile		db	'MBM.144',0



fdbs_code:	; FDu[gR[h (0000:7c00hփ[h)
	_skip
FdcSign		db	'MBM R',VER,': Loading...',0
	_endif
		mov	ax, 1000h
		mov	ds, ax
		mov	es, ax
		mov	ss, ax
		mov	sp, ofs NameBuffer

		mov	bx, 7c00h+ ofs FdcSign-fdbs_code ; NbZ[W
	_while
		mov	al,cs:[bx]
		inc	bx
	 _break e, al, 0
		pusha
		mov	bx,7
		$VIDEO  0eh
		popa
	_rept

	_while
	 _while
		mov	bx, 0100h ; FDC[W{̂̃[h(1000:0100`)
		mov	cx, 0002h ; Cy:0, Sc:2
		mov	dx, 0000h ; Hd:0, Dr:0
		$DISK  2, 17	  ; Read
	 _rept c
		mov	bx, 0100h+512*17
		mov	cx, 0001h ; Cy:0, Sc:1
		mov	dx, 0100h ; Hd:1, Dr:0
		$DISK  2, 18	  ; Read
	_rept c
		mov	FdMode, 1
		db	0eah			; jmp 1000:fdmain
		dw	ofs fdmain, 1000h	; /

fdbs_end:



;-----------------------------------------------;
; ZN^EC[W쐬 (DOS[h)

create_bin:
		MAKE_FILE 0, <ofs BinFile2>	;{̏o
		jc	exit
		WRITE_FILE ax, <ofs mbm_loader>, 512*N_MBM
		jc	exit
		CLOSE_FILE <>

		mov	bp, ofs MbrParms
		FIL_STR  <bp>,<type Parms>, 0
		mov	si, ofs mbm_loader	;{sumvZ
		clr	ax			;
	   _repeat 512/2*N_MBM			;
		add	ax, [si]		;
		incw	si			;
	   _loop				;/
		mov	[bp].MpMbmSum, ax	;MBM sum
		mov	[bp].MpMbmSect, 4	;MBM start sector
		mov	[bp].MpExp, -1		;Pre-expand off

		MAKE_FILE 0, <ofs BinFile1>	;MBRo
		jc	exit
		WRITE_FILE ax, <ofs mbr_code>, <ofs mbr_end-mbr_code>
		jc	exit
		WRITE_FILE <>, <ofs MbrParms>, <type Parms>
		jc	exit
		CLOSE_FILE <>

		PRINT  MesOK
		jmp	exit


BinFile1	db	'MBM_S1.BIN',0
BinFile2	db	'MBM_S4.BIN',0


;-----------------------------------------------;
; FDu[gECEj[ (FD[h)

fdmain:
		PRINT  StrFdUsage
		mov	bx, ofs Pool
		 call	get_str
		 call	get_value
		jc	fdmain
		mov	al, cl
	_case e, al, 1
		jmp	fdrun_mbm	; u[g}l[Ws
	_case e, al, 2
		 call	editor		; GfB^N
		 call	clr_screen
	_case e, al, 3
		 call	install		; CXg[ (Ol)
	_case e, al, 4
		 call	install_high	; CXg[ (l)
	_case e, al, 5
		 call	clr_screen	; u[gV[PXĎs
		int	19h
	_endc
		jmp	fdmain



;-----------------------------------------------;
; u[gE}l[WN (FD[h)

fdrun_mbm:
		mov	si, ofs mbm_loader
		clr	ax
		mov	es, ax
		mov	di, 800h
		mov	cx, N_MBM*512/2
		rep movsw
		FIL_STR <ofs MainParms>, <type Parms>, <>
		mov	ds, ax
		mov	ss, ax
		mov	sp, ofs NameBuffer
		mov	MenuValue, ax		; MenuValue = 0
		mov	MainParms.MpExp, -1	; Pre-expand off
		mov	MainParms.MpFlags, BIT1	; Help ON
		mov	MbmTitle[0], 2fh	; Green title bar
		set	FdBoot			; MBM at FD mode
		db	0eah			; jmp 0:mbm_loader
		dw	ofs mbm_loader, 0	; /



;-----------------------------------------------;
; MBMCXg[ (DOS[h/FD[h)

install_high:	; CXg[ (gbNŌl)
		push	1
	_skip
install:	; CXg[ (gbN擪l)
		push	0
	_endif

	_if e, FdMode, 0
	 _if e,	Argc, 2
		mov	bx, Argv[2]
		cjne	ah, 0, usage
		mov	al, [bx]
		sub	al, '0'
		cja	al, 3, usage
	 _else
		mov	al, 0
	 _endif
	_else
	 _while
		PRINT StrDrvNum
		mov	bx, ofs Pool
		 call	get_str
		 call	get_value
	  _cont c
	 _rept ae, cx, 3
		mov	al, cl
	_endif
		push	ax
		add	al, 80h
		mov	Drive, al
		PRINT MesInst
		pop	ax
		add	al, '0'
		 call	disp_chr
		PRINT  MesCont
		mov	bx,ofs Pool
		 call	get_str
		cmp	Pool, 'Y'
		pop	cx
		jc	exit
		PRINT  MesWrite		; Install into 1st drive
	_if e, cx, 0
		 call	install_mbm_low
	_else
		 call	install_mbm_high
	_endif
		jc	io_err
		PRINT  MesOK
		jmp	exit


get_str_cont:
		clr	cx
	_while
		mov	al,[bx]
	 _break e, al, 0
		 call	disp_chr
		inc	bx
		inc	cx
	_rept
get_str:
	_while
		$KEY 0
	 _break e, al,CR
	 _if ae, al,' '
		 call	disp_chr
		mov	[bx],al
		inc	bx
		inc	cx
	 _else
	  _if e, al,BS
	   _skip cxz
		 call	disp_chr
		dec	bx
		dec	cx
	 ___endif
	_rept
		mov	bptr[bx],0
		 call	disp_chr
		PRINT  CrLf
		ret


disp_str:	; String output on Non-DOS envilonment
	_while
		lodsb
	 _break e, al, 0
		 call	disp_chr
	_rept
		ret


disp_chr:	pusha
		mov	bx,7
		$VIDEO  0eh
		popa
		ret


io_err:		PRINT	StrIOerr
exit:
	_if e, FdMode, 0
		$DOS  4ch,0
	_endif
		ret


;-------------------------------------------------------------------------;
; u[gE}l[W{́iSector 4..j

		org	800h		; MBRR[hɂ0:800hɃ[h
mbm_loader:
		call	beep
		call	beep
		call	beep
		clr	al
	_while
		$KEY 0
	_rept e, al, 0
		CAPS	al
	  _if	e, al, 'L'		;'l' or 'L' key ? if other skip next
	  	 call	ct6b		;<<<--- added by GarakutaEn
	  _endif
		 call	search_disk	; ̃T[`
		 call	chk_preexp	; 
		 call	chk_ovr		; /
	_if c,  <call	load_ext>	; gR[h̃[h
		$VIDEO  0, 3
		 call	init_screen
	 _if c, <call	wait_timer>
		 call	load_go
	 _endif
	_else
	 _if nc,<call	exec_ext>	; gR[hs
		 call	load_go
	 _endif
		 call	init_screen
	_endif

		push	ofs $		; NI[v
		$KEY  0
		mov	bx, ofs BootCmdTable
		 call	scan_cmd
		je	bx
	_skip b,<sub	al,'1'>
		clr	ah
	 _skip ae, ax, MenuItems
		mov	MenuValue, ax
		jmp	rfsh_menu
	__endif
		ret


BootCmdTable	dw	000dh, ofs load_go		; Enter
		dw	0020h, ofs chg_default		; SP, Fix to default
		dw	3b48h, ofs chg_help		; F1/H, Change help
		dw	3c52h, ofs chg_name		; F2/R, Rename menu item
		dw	3d54h, ofs chg_timer		; F3/T, Change timer
		dw	3e56h, ofs show_verbose		; F4/V, Be verbose
		dw	3f41h, ofs chg_active		; F5/A, Activate/Deactivate
		dw	4042h, ofs chg_bootmode		; F6/B, Change boot mode
		dw	414dh, ofs mask_partition	; F7/M, Change mask
		dw	424bh, ofs chg_maskmode		; F8/K, Change auto mask
		dw	4400h, ofs edit_partition	; F10, Editor
		dw	4d04h, ofs expand_drv		; /^D, Expand drive
		dw	4b13h, ofs shrink_drv		; /^S, Close drive
		dw	5018h, ofs cmd_down		; /^X, Up cursor
		dw	4805h, ofs cmd_up		; /^E, Down cursor
		dw	5300h, ofs cmd_restart		; Del, Retry boot sequence
		dw	4f00h, ofs cmd_off		; End, Power off
		db	-1

;++++++++++++++++++++++++++++++++++++++
;Added for TFT-LCD of PC110
;put 'l' or 'L'key after 3 beeps
;if other key will be skip this routine
;
;write TFT value to registers
;	with code display
ct6b:	pusha
	lea	si,tftbl
wtrgr:	mov	al,byte ptr cs:[si]
	cmp	al,0ffh
	je	wtrge
	mov	dx,3d6h
	out	dx,al
	mov	al,byte ptr cs:[si+1]
	mov	dx,3d7h
	out	dx,al
	add	si,2
	jmp	wtrgr
wtrge:	popa
	ret
;----------------------------
;TFT parameter table
tftbl:	db	06h,0c0h	;No color reduction
	db	19h,00h		;Alt Hsync Start(CR04)
	db	1ah,02h		;Alt Hsync End(CR05)
	db	1bh,7fh		;Alt H Total(CR00)
	db	1ch,70h		;Panel Width
	db	45h,80h		;Soft Flag7 on ! <--- this is marking using
	db	4fh,0c6h	;LP/M change to DE
	db	50h,00h		;Clock divide, Dither, FRC
	db	51h,0c4h	;change to LCD on 4h !!!
	db	53h,00h		;for FRC
	db	54h,0c8h	;clock divid (c0h=25MHz?)
	db	55h,0e5h	;H Auto Centering
	db	56h,10h		;H Left Border = Right shift
	db	6ch,1ch		;drive power x2
	db	0ffh,0ffh
;table end
;++++++++++++++++++++++++++++++++++++++

;-----------------------------------------------;
; SP L[

chg_default:	; ftHgN̕ύX
		 call	chk_fdboot
		 call	chk_noitem
		 call	read_mbr0
		mov	ax, MenuValue		;Fix/Release default item
		cmp	ax, MainParms.MpDef	;
		mov	al, MainParms.MpFlags	;
	_if e		 			;
		xor	al, BIT2		;
	_else					;
		set	al, BIT2		;
	_endif					;
		mov	MainParms.MpFlags, al	;
		mov	MbrParms.MpFlags, al	;/
		 call	set_default
		 call	write_mbr0
		jmp	rfsh_menu

set_default:
		mov	ax, MenuValue		;Default
		mov	MainParms.MpDef, ax	;
		mov	MbrParms.MpDef, ax	;
		imul	bp, ax, 4		;Pre-expand?
		cmp	MenuTable[bp].0, 80h	;
		mov	ax, -1			;
	@if ae,<mov	ax, Expand>		;
		mov	MbrParms.MpExp, ax	;/
		ret

;-----------------------------------------------;
;  L[

expand_drv:	; _hCuWJ
	_skip ne, <bptr Expand>, -1		;WJ?
		 call	chk_type		;Logical?
	 _skip ae, ah, 4			;FDD?
		 call	search_extend
	  _skip e, <bptr Expand>, -1
		 call	init_screen
	___endif
		ret

;-----------------------------------------------;
;  L[

shrink_drv:	; _hCuk
		 call	chk_type
	_skip ne, ax, Expand
		 call	search_disk
		 call	chk_ovr
		 call	init_screen
	_endif
		ret

;-----------------------------------------------;
; F1/H L[

chg_help:	; wv\̕\
		 call	read_mbr0
		mov	al, MainParms.MpFlags
		xor	al, BIT1
		mov	MainParms.MpFlags, al
	_skip e, FdBoot, ON
		mov	MbrParms.MpFlags, al
		 call	write_mbr0
	_endif
		jmp	init_screen

;-----------------------------------------------;
; F2/R L[

chg_name:	; 於ύX
		 call	chk_fdboot
		 call	chk_noitem
		imul	bp, MenuValue, 4
		mov	bl, MenuTable[bp].1
		and	bx, 3
		cjne	Installed[bx], 1, beep
		push	bx
		 call	get_nameptr
		push	ax
		imul	di, MenuValue, 80
		add	di, 80*2+7
		test	MainParms.MpFlags, BIT0
	@ifn z,<add	di, 15>
		cmp	Drives, 1
	@if a, <add	di, 2>
		cmp	MenuTable[bp].0, 80h
	@if ae,<add	di, 2>
		 call	input_strn
		pop	di, ax
	_skip z
		MOV_MEM <ofs Pool>, <>, <type NameItem>
		 call	save_name
	_endif
		jmp	init_screen

;-----------------------------------------------;
; F3/T L[

chg_timer:	; ^C}[lύX
		 call	chk_fdboot
		 call	set_csrhome
		mov	si, ofs ChgTimer
		 call	put_stra
		clr	ax
	_skip z,<call	input_strn>
		 call	get_value
		push	cx
		 call	read_mbr0
		pop	MbrParms.MpTime
		 call	write_mbr0
	_endif
		jmp	init_screen

;-----------------------------------------------;
; F4/V L[

show_verbose:	; ڍו\ ON/OFF
		 call	read_mbr0
		mov	al, MainParms.MpFlags
		xor	al, BIT0
		mov	MainParms.MpFlags, al
	_skip e, FdBoot, ON
		mov	MbrParms.MpFlags, al
		 call	write_mbr0
	_endif
		 call	search_disk
		 call	chk_ovr
		jmp	init_screen

;-----------------------------------------------;
; F5/A L[

chg_active:
		 call	chk_noitem
	_if nz, <test	MainParms.MpFlags, BIT0>
		 call	chk_type
	 _skip ae, ah, 4	; FDDȂ牽Ȃ
		push	ax
		or	ah, 80h
		mov	dl, ah
		 call	read_mbr2
		pop	bx
		mov	dh, bh
		clr	bh
		shl	bx, 4
		mov	al, MbrPartition[bx].SystemID
		 call	chk_dos
	  _skip a
		clr	ah
		cmp	MbrPartition[bx].BootID, 80h	;Change
		mov	al, ah				;
	   @if b,<mov	al, 80h>			;/
		mov	di, ofs MenuTable		;Update Memory
	   _repeat MenuItems				;
	    _if e, [di].1, dh				;
	     _if be, bptr[di].0, 3			;
		mov	[di].3, ah			;
	    __endif					;
		add	di, 4				;
	   _loop					;
		mov	MenuTable[bp].3, al		;/
		mov	MbrPartition[0].BootID, ah	;Update Disk
		mov	MbrPartition[16].BootID, ah	;
		mov	MbrPartition[32].BootID, ah	;
		mov	MbrPartition[48].BootID, ah	;
		mov	MbrPartition[bx].BootID, al	;/
		 call	write_mbr2
		 call	rfsh_menu
	___endif
		ret

;-----------------------------------------------;
; F6/M L[

mask_partition:	; DOS̃}XN
		 call	chk_noitem
	_if nz, <test	MainParms.MpFlags, BIT0>
		 call	chk_type
	 _skip ae, ah, 4	; FDDȂ牽Ȃ
		push	ax
		or	ah, 80h
		mov	dl, ah
		 call	read_mbr2
		pop	bx
		clr	bh
		shl	bx, 4
		mov	al, MbrPartition[bx].SystemID
		 call	chk_dos
	  _skip a
		xor	al, 10h
		mov	MbrPartition[bx].SystemID, al
		mov	MenuTable[bp].2, al
		 call	write_mbr2
		 call	rfsh_menu
	___endif
		ret

;-----------------------------------------------;
; F7/B L[

chg_bootmode:	; Boot mode ̐؂芷
		 call	chk_noitem
	_if nz,<test	MainParms.MpFlags, BIT0>
		 call	chk_fdboot
		 call	chk_type
	 _skip ae, ah, 4	; FDDȂ牽Ȃ
		 call	read_mbr0
		 call	get_boot_mask
		inc	al
		@32 <mov bx, MainParms.MpBoot>
		 call	store_maskbit
		@32 <mov MainParms.MpBoot, ax>
		@32 <mov MbrParms.MpBoot, ax>
		 call	write_mbr0
		 call	rfsh_menu
	__endif
		ret

store_maskbit:	@32 <mov dx, 3>,0
		@32 <and ax, dx>
		@32 <not dx>
		@32 <rol ax, cl>
		@32 <rol dx, cl>
		@32 <and dx, bx>
		@32 <or ax, dx>
		ret

;-----------------------------------------------;
; F8/K L[

chg_maskmode:	; Mask mode ̐؂芷
		 call	chk_noitem
	_if nz,<test	MainParms.MpFlags, BIT0>
		 call	chk_fdboot
		 call	chk_type
	 _skip ae, ah, 4	; FDDȂ牽Ȃ
		 call	read_mbr0
		 call	get_boot_mask
		mov	al, ah
		inc	al
		and	al, 3
		@32 <mov bx, MainParms.MpMask>
		 call	store_maskbit
		@32 <mov MainParms.MpMask, ax>
		@32 <mov MbrParms.MpMask, ax>
		 call	write_mbr0
		 call	rfsh_menu
	__endif
		ret


;-----------------------------------------------;
; F10 L[

edit_partition:	; e[uGfB^
		 call	chk_fdboot
		 call	editor
		 call	search_disk
		 call	chk_ovr
		jmp	init_screen

;-----------------------------------------------;
;  L[

cmd_down:
		mov	ax, MenuValue
		inc	ax
	_if b, ax, MenuItems
		mov	MenuValue, ax
		 call	rfsh_menu
	_endif
		ret

cmd_up:
	_if ne, MenuValue, 0
		dec	MenuValue
		 call	rfsh_menu
	_endif
		ret

;-----------------------------------------------;
; Del L[

cmd_restart:
		 call	clr_screen
		int	19h
		ret

;-----------------------------------------------;
; End L[

cmd_off:
		clr	bx
		$SYSTEM  53h,1
		clr	bx
		mov	cx,101h
		$SYSTEM  53h,0eh
		mov	bx,1
		mov	cx,1
		$SYSTEM  53h,0fh
		mov	bx,1
		mov	cx,1
		$SYSTEM  53h,8
		mov	bx,1
		mov	cx,3
		$SYSTEM  53h,7
		ret

;-----------------------------------------------;
; gj[ǂݍ

load_ext:
		mov	Drive, 80h
		 call	get_drvparm
		jc	le_err
		$KEY  2				; Ctrl pressed?
		and	ax, BIT2		;
		jnz	le_err			;/
		cje	Lba, al, le_err		; No LBA ?
		cje	MenuItems, ax, le_err	; No menu item?

		 call	cyl_size
		@32 <mov dx, MainParms.MpCyl>
		@32 <inc dx>
		@32 <mul dx>
		@32 <clr cx>
		mov	cx, MainParms.MpSects
		jcxz	le_err
		cjae	cx, 1200, le_err
		@32 <sub ax, cx>
		@32 <mov di, ax>
		movseg	es, 800h
		clr	bx
	_repeat
		push	cx
		@32 <push 0>
		@32 <push di>
		push	es,bx, 1, 16
		mov	si, sp
		mov	dl, 80h
		$DISK  42h
		lea	sp, [si].16
		jc	le_err2
	 _repeat 512/2
		mov	ax, es:[bx]
		sub	MainParms.MpSum, ax
		incw	bx
	 _loop
	 _if e, bx, 0
		mov	ax, es
		add	ax, 1000h
		mov	es, ax
	 _endif
		pop	cx
		@32 <inc di>
	_loop
		movseg	es, cs
		cjne	MainParms.MpSum, 0, le_err
		clc
		ret
le_err2:	pop	ax
le_err:		movseg	es, cs
		stc
		ret



;-----------------------------------------------;
; gj[s

exec_ext:
		clr	bp
		mov	di, ofs MbrBuffer
	_repeat MenuItems
		push	cx
		 call	get_nameptr
		xchg	ax, si
		cmp	bptr[si], 0
	 @if e,<mov	si, ax>
		MOV_MEM <>, <>, 32
		add	bp, 4
		pop	cx
	_loop
		mov	ax, MenuValue
		mov	cx, MenuItems
		mov	dx, MainParms.MpTime
		mov	bx, ofs MbrBuffer
		db	09ah		;CALL 0800:0000
		dd	08000000h	;/
		movseg	ds, cs
		movseg	es, cs
		push	ax
		$VIDEO  0, 3
		pop	ax
		cld
		cmp	ax, MenuItems
		cmc
	@ifn c,<mov	MenuValue, ax>
		ret


;-----------------------------------------------;
; ANeBuݒ^[h^s

load_go:	call	chk_noitem
		$KEY 2			;R-Shift Key?
		test	al, BIT0	;
		jnz	load_pbr	;/

		; ftHgύX
	_skip e, FdBoot, ON
	 _skip nz, <test MainParms.MpFlags, BIT2>
		 call	read_mbr0
		 call	set_default
		 call	write_mbr0
	__endif

		; ANeBuݒ
		 call	get_boot_mask			;Get boot mode
		mov	wptr BootMode[0], ax		;/
	_skip nz, <test	wptr MenuTable[bp].0, 0480h>	;FDD or Logical?
		@32 <mov di,MainParms.MpMask>
		mov	dl, 80h
		clr	cx
		mov	cl, Drives
	 _repeat
		push	cx
		 call	read_mbr2
		mov	bx, wptr MenuTable[bp].0;bx = target drive, partition
		or	bh, 80h			;/
		clr	ax
	 _case e, BootMode[0], 0
		inc	ax
	 _case be, BootMode[0], 2
	  _if e, dl, bh
		inc	ax
	  _endif
	 _endc
		mov	BootMode[2], al
		pusha
		mov	dh, dl
		clr	dl
		mov	ax, di
		mov	ah, al		;ah = mask modes
		mov	bp, ofs MbrPartition
	  _repeat 4
		clr	al			;Skip if activate enabled
	   _skip e, al, BootMode[2]		;
	    _if e, dh, 80h			; If boot from 2nd and exist dummy, 
	     _if ne, dh, bh			; activate it
	      _if e, [bp].SystemID, 0f0h	; 
		mov	al, 80h			; 
	    ___endif				; /
	    _if e, dx, bx			; Activate boot partition
		mov	al, 80h			; 
	    _endif				; /
		mov	[bp].BootID, al		; Update boot ID
	   _endif				;/
	   _skip z, <test ah, 3>		;Mask group
		mov	al, [bp].SystemID	;
		push	ax			;
		 call	chk_dos			;
		pop	ax			;
	    _skip a				;
		res	[bp].SystemID, BIT4	;
		mov	al, ah			;
		and	al, 3			;
	     _skip e, al, BootMode[1]		;
		set	[bp].SystemID, BIT4	;
	   ___endif				;/
		shr	ah, 2
		add	bp, type PartitionTbl
		inc	dl
	  _loop
		popa
		 call	write_mbr2		; Update partition table
		inc	dl
		@32 <shr di,8>
		pop	cx
	 _loop
	_endif

load_pbr:	; PBR̃[h
		mov	ErrRetry, 5
lg_retry:	imul	bp, MenuValue, 4
		mov	ax, wptr MenuTable[bp].0
	_if b, ah, 4
		or	ah, 80h		;Load target partiotion table
		mov	Drive, ah
		 call	get_pbrlst
		MOV_MEM ax, 700h, <type PartitionTbl>	;Save table
		mov	bp, ax
		 call	get_drvparm
		jc	lg_err
		 call	read_pbr
	_else
		mov	dl, 0		; FDD#0
		 call	read_mbr2
	_endif
		jc	lg_err
		cjne	MbrSign, 0aa55h, lg_invalid
		push	bx

		; hCuXbvR[h̃CXg[
	_skip be, dl, 80h		 	; FDD/1ڂ̏ꍇ͂Ȃ
	 _skip ae, BootMode[0], 2	 	; Mode >= 2 ̏ꍇ͂Ȃ
		mov	al, [bp].SystemID	; DOSnȊOȂ炵Ȃ
		mov	di, ofs DosTable	;
		mov	cx, len DosTable	;
		cld				;
		repne scasb			;
	  _skip ne				; /
		mov	BiosSwap, dl
		@32 <mov ax, 0006h>,31h	;mov eax, 00310006h
		@32 <xchg ax, cs:[13h*4]>
		@32 <mov wptr BiosLink, ax>
		MOV_MEM <ofs bios_hook>, 310h, <ofs bh_end - ofs bios_hook>
		mov	dl, 80h
	___endif

		 call	clr_screen		;ʃNA
	 _while					;R-ShiftJ҂
		$KEY 2				;
		test	al, BIT0		;
	 _rept nz				;/
		mov	si, 700h		;Table entry
		clr	bp			;PBRɐڂ
		ret				;/


lg_err:		dec	ErrRetry		;DISKG[
	_skip z
		$DISK 0				;hCuZbgăgC
		jmp	lg_retry		;/
	_endif
		mov	si, ofs DiskErr		;Disk error
	_skip
lg_invalid:	mov	si, ofs InvalidPbr	;Invalid PBR
	_endif
		push	si
		 call	init_screen
		 call	set_csrhome
		pop	si
		 call	put_stra
		jmp	wait_clr



DosTable	db	1, 4, 6, 7, 0bh, 0ch, 0eh, 11h, 14h, 16h, 17h, 1bh, 1ch, 1eh	; DOSnOS

bios_hook:	; DISK SWAP code
BiosLink	dd	?	; 0 link
BiosSwap	db	?	; 4 swap
		db	?	; 5 save
		mov	cs:[5],dl ; 6 code
	_case e, dl,80h
		mov	dl, cs:[4]
	_case e, dl,cs:[4]
		mov	dl, 80h
	_endc
		pushf
	_skip e, ah,8h
	 _skip e, ah,15h
		 call	dwptr cs:[0]
		mov	dl, cs:[5]
		retf	2
	__endif
		 call	dwptr cs:[0]
		retf	2
bh_end:



;-----------------------------------------------;
; ʂ̏^j[ڕ\

init_screen:
		 call	clr_screen		; ʃNA
		 call	hide_csr		; Csr off
		mov	si, ofs MbmTitle	; ^Cg
		clr	di
		 call	put_stra
		and	ah, 0f7h		; wv
	_if z,<test MainParms.MpFlags, BIT1>
		 call	put_str
	_else
		mov	si, ofs StrSpace[0]
		 call	put_str
		mov	di, 2*80 + 56
		mov	si, ofs HelpStr
	 _while
		push	di
		mov	ah, 0fh
		 call	put_str
		mov	ah, 07h
		 call	put_str
		pop	di
		add	di, 80
	 _rept ne, [si], al
	_endif

rfsh_menu:	; j[̕\
		mov	di, 80*2 + 1	; j[
		clr	dx
	_repeat MenuItems
	 _break cxz
		pusha
		imul	bp, dx, 4
		cmp	dx, MenuValue		;On the cursor ?
		mov	al, 07h			;
	 @if e,<mov	al, 0f0h>		;
		mov	ChrAttr, al		;/
		mov	ax, 0f20h		;Default mark
	 _if e, dx, MainParms.MpDef		;
	  _if nz,<test MainParms.MpFlags, BIT2>	;
		mov	al, '*'			;
	 __endif				;
		 call	put_chr			;/
		mov	al, dl			;Item #
		inc	al			;
		push	dx			;
		mov	dl, 2			;
		mov	dh, ChrAttr		;
		 call	put_num1		;
		pop	dx			;
		mov	al, '.'			;
		 call	put_chrf		;
		mov	al, ' '			;
		 call	put_chrf		;/
	 _if ae, Drives,2
	  _skip ae, MenuTable[bp].0, 80h	;Drive #
		mov	al, MenuTable[bp].1	;
		add	al, '0'			;
		cmp	al, '4'			;
	   @if e,<mov	al, 'F'>		;
		mov	si, ofs	StrDrv		;
		mov	[si], al		;
	  _else					;
		mov	si, ofs StrSpace[6]	;
	  _endif				;
		 call	put_strf		;/
	 _endif
	 _if nz,<test MainParms.MpFlags, BIT0>	; Vervose ?
	  _if e, MenuTable[bp].1, 4
	   _repeat 15				;FDD is no status
		mov	al, ' '			;
		 call	put_chrf		;
	   _loop				;/
	  _else
		mov	si, ofs StrModes
		xchg	dx, MenuValue		;Get boot_mask mode
		push	bp			;
		 call	get_boot_mask		;
		pop	bp			;
		xchg	dx, MenuValue		;
		add	ax, '00'		;
		mov	cx, ax			;
		mov	ax, '--'		;Activated/Boot mode
		test	MenuTable[bp].3, BIT7	;
	   @ifn z,<mov	al, 'a'>		;
		cmp	cl, '0'			;
	   @ifn e,<mov	ah, cl>			;
		mov	[si].1, ax		;/
		mov	al, MenuTable[bp].2	;Masked/Mask mode
		 call	chk_dos			;
		mov	ax, '--'		;
	   @if e,<mov	al, 'm'>		;
		cmp	ch, '0'			;
	   @ifn e,<mov	ah, ch>			;
		mov	[si].3, ax		;/
		 call	put_strf
		mov	ax, wptr MenuTable[bp].0;Partition size
		 call	get_pbrlst		;
		mov	bx, ax			;
		@32 <mov ax, [bx].PartitionSize>;
		 call	put_size		;/
		mov	al, MenuTable[bp].2	;Partition ID
		 call	cv_hex			;
		mov	[si].1, ax		;/
		 call	put_strf
	  _endif
	 _endif
	 _if ae, MenuTable[bp].0, 80h
		mov	al, ' '			;Partition name (logical)
		 call	put_chrf		;
		cmp	MenuTable[bp+4].0, 80h	;
		mov	al, 0c3h		;
	  @if b,<mov	al, 0c0h>		;
		 call	put_chrf		;
		mov	al, ' '			;/
	 _else
		mov	dl, MenuTable[bp].2	;Partition name (primary)
		 call	chk_extend		; Expand mark
	 	mov	al, ' '			;
	  @if e,<mov	al, '+'>		;
		mov	bx, wptr MenuTable[bp].0;
		cmp	bx, Expand		;
	  @if e,<mov	al, '-'>		;/
	 _endif
		 call	put_chrf
		 call	get_nameptr
		xchg	ax, si
		cmp	bptr[si], 0
	 @if e,<mov	si, ax>
		 call	put_strf
		popa
		add	di, 80
		inc	dx
		dec	cx
	_rept
		ret



put_size:	mov	cx, 2048		;sectors/2048 = Mbyte
		 call	div_eax
	_if ae, cx, 2048/2
		@32 <inc ax>
	_endif
		mov	dh, ChrAttr
		@32 <cmp ax, 36809>,1	;cmp eax, 102345
	_case ae
		mov	cx, 1024
		 call	div_eax
	 _if ae, cx, 1024/2
		@32 <inc ax>
	 _endif
		mov	dl, 5
		 call	put_num2
		mov	al, 'G'
	_case ae, <@32 <cmp ax, 10000>,0>
		push	dx
		@32 <mov dx, 10>,0
		@32 <mul dx>
		pop	dx
		mov	cx, 1024
		 call	div_eax
	 _if ae, cx, 1024/2
		@32 <inc ax>
	 _endif
		mov	cx, 10
		 call	div_eax
		mov	dl, 3
		 call	put_num2
		mov	al, '.'
		 call	put_chrf
		mov	al, cl
		mov	dl, 1
		 call	put_num1
		mov	al, 'G'
	_default
		mov	dl, 5
		 call	put_num2
		mov	al, 'M'
	_endc
		jmp	put_chrf

put_size2:
		mov	ChrAttr, 0fh
		 call	put_size
		mov	al,'B'
		jmp	put_chrf



wait_timer:	; N^C}[\iCF=set on break, clr on time outj
	_skip e, MainParms.MpTime, 0
		 call	set_csrhome
		mov	bp, 10
	 _while
		push	di
		mov	si, ofs TimerStr
		 call	put_stra
		sub	si, bp
		add	si, 10
		mov	ah, 07h
		 call	put_str
		 call	put_stra
		pop	di
		mov	cx, MainParms.MpTime
		 call	wtime
	  _break nz
	 _rept nz,<dec	bp>
		pushf
		 call	clr_csrline
		popf
	 _if z
		stc
		ret
	 _endif
	_endif
		clc
		ret


set_csrhome:	; J[\z[ʒuij[̉j
		imul	di, MenuItems, 80
		add	di, 80*3 +3
		ret


wait_clr:	; 1.5b҂ăbZ[WsNA
		mov	cx, 16
		 call	wtime
clr_csrline:	; bZ[WsNA
		 call	set_csrhome
		mov	cx, 49
		jmp	clr_block


wtime:		; cx * 100ms ҂iL[͂Œfj
	 _repeat
		push	cx
		mov	cx, 1
		mov	dx, 8000h
		$SYSTEM  86h
		$KEY  1
		pop	cx
	 _loop z
		ret


;-----------------------------------------------;


get_boot_mask:	; u[g[h擾  Call:MenuValue, Ret:ah=mask mode, al=boot mode
		 call	chk_type
		mov	cx, ax
		shl	ch, 2
		or	cl, ch
		shl	cl, 1
		@32 <mov ax,MainParms.MpMask>
		@32 <shr ax, cl>
		mov	ch, al
		@32 <mov ax,MainParms.MpBoot>
		@32 <shr ax, cl>
		mov	ah, ch
		and	ax, 0303h
		ret


get_nameptr:	; 於|C^擾  call: bp=table offset, ret: ax -> table ofs, si -> name str
	_if ae, MenuTable[bp].1, 4	; FDD?
		mov	ax, ofs NameBuffer[type NameItem*16]
		mov	si, ofs StrFdd
		ret
	_endif
		mov	ah, MenuTable[bp].2
		 call	get_osname
		mov	ax, wptr MenuTable[bp].0
		push	ax
	_if b, al, 80h	; root?
		and	ax, 3
		imul	dx, ax, type NameItem
	_else
		and	ax, 1fh
		imul	dx, ax, type NameItem
		add	dx, 512
	_endif
		pop	ax
		and	ax, 300h
		shl	ax, 2
		add	ax, dx
		add	ax, ofs NameBuffer
		ret


chk_dos:	; DOS`FbN(NC/NZ:Non-DOS, CY/NZ:Unmasked, NC/ZR:Masked)
		cjae	al, 20h, cd_ndos
		cjae	al, 14h, cd_mdos
		cje	al, 11h, cd_mdos
		ja	cd_ndos
		cje	al, 10h, cd_ndos
		cjae	al, 04h, cd_udos
		cja	al, 01h, cd_ndos
		je	cd_udos
	cd_ndos:or	ah, -1	; Non-DOS
		ret
	cd_mdos:clr	ah	; DOS/Masked
		ret
	cd_udos:or	ah, -1	; DOS/Unmasked
		stc
		ret


get_pbrlst:	; AH(drv),AL(partition) PbrList̃|C^AXɓ
	  _if ae, al, 80h
		and	ax, 7fh
		shl	ax, 4	;*16
		add	ax, ofs PbrList[type PartitionTbl*16]
	  _else
		and	ah, 7fh
		shl	al, 6	;(ah*4+al)*16
		shr	ax, 2	;/
		add	ax, ofs PbrList
	  _endif
		ret


chk_fdboot:	; FDu[g̏ꍇG[Ŗ߂
	_if ne	FdBoot, 0
		pop	ax
		jmp	beep
	_endif
		ret


chk_ovr:	; ڔԍ̃I[o[t[␳
		mov	ax, MenuValue
	_if ae, ax, MenuItems
		clr	ax
		mov	MenuValue, ax
		mov	MainParms.MpTime, ax
	_endif
		ret


chk_noitem:	; ږ̏ꍇ߂
	_if e, MenuItems, 0
		pop	ax
	_endif
		ret


chk_type:	; _hCȕꍇ߂
		imul	bp, MenuValue, 4
		mov	ax, wptr MenuTable[bp].0
	_if ae, al, 80h
		pop	ax
		mov	ax, 3	;for get_bootmode/maskmode
	_endif
		ret


chk_extend:	; ZR: dl = gID
	 _skip e, dl, 05h
	  _skip e, dl, 0fh
	   _skip e, dl, 15h
	    _skip e, dl, 1fh
		cmp	dl, 85h
	 ____endif
		ret


;-----------------------------------------------;

search_disk:	; ڑhCuƋ̃T[`
		@32 <clr ax>
		@32 <mov wptr Installed, ax>
		FIL_STR <ofs NameBuffer>,1024*4,<>
		FIL_STR <ofs MenuTable>,<size MenuTable>,<>
		mov	bptr Expand,-1	; Expand OFF
		mov	di, ofs PbrList
		clr	bp, bx
		mov	Drives, al
		mov	MenuItems, ax
		mov	Drive, 80h
	_while
		push	bx
		 call	read_mbr
		pop	bx
	 _break c
		mov	si, di
		 call	chk_mbrbuff
		 call	load_name
		xchg	si, di
		mov	ah, Drive
		and	ax, 7f00h
	 _repeat 4
		push	cx
	  _if ne, [si].SystemID, 0		; check valid partition
	   _if z,<test MainParms.MpFlags, BIT0>
	    _if ne, <bptr NameBuffer[bx]>, '*'
		 call	store_addr
	    _endif
	   _else
		 call	store_addr
	  __endif
		mov	cx, type PartitionTbl
		rep movsb
		add	bx, type NameItem
		inc	al
		pop	cx
	 _loop
		add	bx, 1024-type NameItem*4
		inc	Drives
		inc	Drive
	_rept b, Drives,4

		mov	ax, 0400h	; Add FDD item
		test	MainParms.MpFlags, BIT0
	@if z, <cmp	bptr NameBuffer[type NameItem*16], '*'>
	@ifn z,<call	store_addr>
		ret

store_addr:	mov	dl, [si].SystemID
		mov	dh, [si].BootID
		mov	wptr MenuTable[bp].2, dx
		mov	wptr MenuTable[bp].0, ax
		add	bp, 4
		inc	MenuItems
		ret



chk_preexp:	; \WJ̃`FbN
		mov	ax, MainParms.MpExp
		clr	bp
	_repeat MenuItems
	 _break cxz
	 _break e, ax, <wptr MenuTable[bp].0>
		add	bp, 4
	_loop
	_skip cxz
		 call	search_extend
	_endif
		ret



search_extend:	; ǧƃj[ڒǉ (MenuTable[bp] = target item)
		mov	dl, MenuTable[bp].2
		 call	chk_extend
		jne	exd_x
		mov	ax, wptr MenuTable[bp].0
		mov	Expand, ax	;Expand ON
		push	bp
		mov	dl, ah
		or	dl, 80h
		mov	Drive, dl
		shl	al, 6		;bp -> Extend partition
		shr	ax, 2		;
		add	ax, ofs PbrList	;
		mov	bp, ax		;
		 call	get_drvparm
		pop	bx
		jc	exd_x
		@32 <mov ax, [bp].PartitionOfs>
		@32 <mov wptr Pool[0], ax>
		@32 <mov wptr Pool[4], ax>
		mov	di, ofs PbrList[type PartitionTbl*16]
		mov	Pool[8], 0
exd_lp:		pusha
		 call read_pbr
		popa
		jc	exd_x
		cjne  MbrSign, 0aa55h, exd_x
		mov	si, ofs MbrPartition
	_repeat 4
		mov	dl, [si].SystemID
		 call	chk_extend
	 _if e
		@32 <mov ax, wptr Pool[0]>
		@32 <add ax, [si].PartitionOfs>
		@32 <mov [si].PartitionOfs, ax>
		@32 <mov wptr Pool[4], ax>
		mov	bp, si
		jmp	exd_lp
	 _endif
	 _skip e, dl, 0
		cjae	MenuItems, 21, exd_x
		mov	al, Pool[8]
		cjae	al, 17, exd_x
	  _skip nz, <test MainParms.MpFlags, BIT0>
		push	bx
		clr	ah
		imul	bx, ax, type NameItem
		mov	al, bptr Expand[1]
		shl	ax, 10
		add	bx, ax
		cmp	bptr NameBuffer[512][bx], '*'
		pop	bx
	  _endif
	  _skip e
		inc	MenuItems
		add	bx, 4
		pusha
		mov	dl, [si].SystemID
		mov	dh, [si].BootID
		@32 <shl dx, 16>
		mov	dh, bptr Expand[1]
		mov	dl, Pool[8]
		add	dl, 80h
	   _while
		@32 <xchg dx, wptr MenuTable[bx]>
		add	bx, 4
	   _rept b, bx, <size MenuTable>
		@32 <mov ax, wptr Pool[4]>
		@32 <add [si].PartitionOfs, ax>
		mov	cx, type PartitionTbl
		rep movsb
		popa
	  _endif
		add	di, type PartitionTbl
		inc	Pool[8]
	 _endif
		add	si, type PartitionTbl
	_loop
exd_x:		ret




get_drvparm:	; hCup[^擾 (Drive = drive# to be checked)
		FIL_STR <ofs Pool>,100,0
		mov	Lba, al
		mov	dl, Drive		;Read drive parms
		$DISK  8			;
	_skip c					;
		@32 <clr ax>			;
		mov	Heads, dh		;
		mov	ax, cx			;
		xchg	ah, al			;
		shr	ah, 6			;
		inc	ax			;
		@32 <mov Cylinders, ax>		;
		and	cl, 3fh			;
		mov	Sectors, cl		;
		mov	bx, 55aah		; check LBA
		mov	dl, Drive		;
	 _skip c,<$DISK  41h>			;
	  _skip ne, bx, 0aa55h			;
	   _skip z, <test cl, BIT0>		;
		mov	si, ofs Pool		;
		mov	wptr[si], 1ah		;
	    _skip c, <$DISK  48h>		;
		@32 <mov ax,[si].16>		;
		mov	cl, Sectors		;
		 call	div_eax1		;
		mov	cl, Heads		;
		inc	cx			;
		 call	div_eax			;
		@32 <mov Cylinders, ax>		;
		inc	Lba			; /
	 ____endif				;/
		clc
	_endif
		ret



load_name:	; NameTableǂݍ
		pusha
	_skip e, FdBoot, ON
		CMP_STR <ofs MbrBuffer[2]>,<ofs MbmSign>,<len MbmSign>
	 _skip ne
		clr	dx
		mov	dl, Drive
		mov	bx, dx
		and	bl, 3
		mov	Installed[bx], 1
		shl	bx, 10
		add	bx, ofs NameBuffer
		mov	cx, 0002h
		mov	bp, 2
	  _while
		$DISK  2, 1
		push	cx, bx
		 call	sum_name
	   _if ne, ax, [bx]
		pop	di
		push	di
		FIL_STR <>, 512, 0
	   _endif
		pop	bx, cx
		add	bx, 512
		inc	cl
	  _rept nz,<dec	bp>
	__endif
		popa
		ret


save_name:	; NameTable (ax = drive number (0..3))
		pusha
		mov	bx, ax
		shl	bx, 10
		add	bx, ofs NameBuffer
		or	al, 80h
		mov	dx, ax
		push	bx
		 call	sum_name
		mov	[bx], ax
		incw	bx
		 call	sum_name
		mov	[bx], ax
		pop	bx
		mov	cx, 0002h
		$DISK  3, 2
		popa
		ret

sum_name:	mov	ax, 33cch
	_repeat 510/2
		add	ax, [bx]
		incw	bx
	_loop
		ret


read_mbr:	mov	dl, Drive
		jmp	read_mbr2
read_mbr0:	mov	dl, 80h
read_mbr2:	clr	dh
		mov	cx, 0001h
		mov	bx, ofs MbrBuffer
		$DISK  2, 1
		ret


write_mbr:	mov	dl, Drive
		jmp	write_mbr2
write_mbr0:	mov	dl, 80h
write_mbr2:	clr	dh
		mov	cx, 0001h
		mov	bx, ofs MbrBuffer
		$DISK  3, 1
		ret


read_pbr:	; PBRǂݍ (Drive=drive#, bp -> partition item)
		mov	dl, Drive
		mov	bx, ofs MbrBuffer
	 _if e, Lba, 0
		mov	dh, [bp].StartHead
		mov	cx, [bp].StartSectCyl
		$DISK  2,1
		ret
	 _endif
		@32 <push 0>
		@32 <push [bp].PartitionOfs>
		push	es,bx, 1, 16
		mov	si, sp
		$DISK  42h
		lea	sp, [si].16
		ret



cyl_size:				; V_TCY擾
		@32 <clr ax>		; Ret: eax = number of sectors par cyl.
		mov	ah, Sectors	;
		mov	al, Heads
		inc	al
	@ifn z,<mul	ah>
		ret


div_eax1:
		clr	ch
div_eax:
		@32 <push dx>		; Call: eax = dividend
		@32 <clr dx>		;       cx = divisor	
		@32 <shl cx,16>		; Ret:  eax = result
		@32 <shr cx,16>		;
		@32 <div cx>		;       cx = remander
		mov	cx, dx
		@32 <pop dx>
		ret



;----------------------------------------------------------------------------;
; e[uGfB^

editor:		; {ҏW
		 call	scan_drives		;hCu
		mov	Drive, 80h		;ҏWhCu
edit_ce:	 call	clr_screen		;Editʕ`
		mov	si, ofs EditForm	;
		 call	put_stra		;
		 call	hide_csr		;/
edit_chg:	 call	read_mbr_parm
		ret c
		 call	chk_bound
		 call	disp_parm
		 call	rfsh_table
		 call	chk_table

		push	ofs $
		$KEY  0
		mov	bx, ofs EditCmdTable1
	_if ne, InExtend, 0
		mov	bx, ofs EditCmdTable2
	_endif
		 call	scan_cmd
		mov	cx, EditPosY
		mov	dx, EditPosX
		mov	bp, EditItems
		jne	enter_num
		mov	ax, EditOfsY
		jmp	bx



EditCmdTable1	dw	4b13h, ofs move_left	;  ^S
		dw	4d04h, ofs move_right	;  ^D
		dw	4805h, ofs move_up	;  ^E
		dw	5018h, ofs move_down	;  ^X
		dw	4912h, ofs val_up	; PgUp ^R
		dw	5103h, ofs val_down	; PgDn ^C
		dw	3f00h, ofs unformat	; F5
		dw	4200h, ofs save_table	; F8
		dw	4300h, ofs inst_mbm	; F9
		dw	5c00h, ofs clr_ntsign	; Sft-F9
		dw	4400h, ofs chg_drive	; F10
		dw	0020h, ofs tgl_active	; SP
		dw	000dh, ofs enter_extend	; Enter ^M
		dw	001bh, ofs quit_edit	; Esc ^[
		db	-1

EditCmdTable2	dw	4b13h, ofs move_left	;  ^S
		dw	4d04h, ofs move_right	;  ^D
		dw	4805h, ofs move_up	;  ^E
		dw	5018h, ofs move_down	;  ^X
		dw	4912h, ofs val_up	; PgUp ^R
		dw	5103h, ofs val_down	; PgDn ^C
		dw	4200h, ofs save_table	; F8
		dw	5200h, ofs ins_item	; Ins
		dw	5300h, ofs del_item	; Del
		dw	001Bh, ofs quit_edit	; Esc ^[
		db	-1


;----------------------------------------------;
; ҏWR}h


;Pri/Ext
quit_edit:	; ҏWI
		 call	check_changed
	_skip nc
		pop	ax
		cjne	InExtend, 0, edit_ce
	_endif
		ret


;Pri/Ext
move_left:	; ̍
	_skip e, dx, 0
		dec	EditPosX
		 call	rfsh_table
	_endif
		ret


;Pri/Ext
move_right:	; E̍
	_skip ae, dx, 2
		inc	EditPosX
		 call	rfsh_table
	_endif
		ret


;Pri/Ext
move_up:	; ̋
		add	ax, cx
	_skip z
		mov	di, cx
		mov	si, di
		dec	si
		 call	order_swap
	  _if e, EditPosY, 0
		dec	EditOfsY
	  _else
		dec	EditPosY
	  _endif
		 call	rfsh_table
	_endif
		ret


;Pri/Ext
move_down:	; ̋
		add	ax, cx
	_if e, InExtend, 0
		inc	ax
	_endif
	_skip ae, ax, bp
		mov	di, cx
		mov	si, di
		inc	si
		 call	order_swap
	 _if ae, EditPosY, 6
		inc	EditOfsY
	 _else
		inc	EditPosY
	 _endif
		 call	rfsh_table
	_endif
		ret


;Pri/Ext
val_up:		; l++
		add	ax, cx
		imul	bx, ax, 16
		add	bx, ofs EditTbl
		@32 <mov ax, [bx].0>
		@32 <mov cx, [bx].4>
	_case e, dx, 1			; Jn
		@32 <inc ax>
		@32 <cmp ax, Cylinders>
		ret ae
		@32 <cmp ax, cx>
	 @if a,<@32 <mov cx, ax>>
		set	EditFlags, F_CC
	_case e, dx, 2			; I
		@32 <inc cx>
		@32 <cmp cx, Cylinders>
		ret ae
		set	EditFlags, F_CC
	_default			; VXe
		inc	bptr[bx].8
	 @if z,<mov	bptr[bx].9, dl>
		set	EditFlags, F_IC
	_endc
		@32 <mov [bx].0, ax>
		@32 <mov [bx].4, cx>
		 call	rfsh_table
		jmp	chk_table


;Pri/Ext
val_down:	; l--
		add	ax, cx
		imul	bx, ax, 16
		add	bx, ofs EditTbl
		@32 <mov ax, [bx].0>
		@32 <mov cx, [bx].4>
	_case e, dx, 1			; Jn
		@32 <sub ax,1>,0
		ret b
		set	EditFlags, F_CC
	_case e, dx, 2			; I
		@32 <sub cx,1>
		ret b
		@32 <cmp cx,ax>
	 @if b,<@32 <mov ax,cx>>
		set	EditFlags, F_CC
	_default			; VXe
		dec	bptr[bx].8
	 @if z,<mov	bptr[bx].9, dl>
		set	EditFlags, F_IC
	_endc
		@32 <mov [bx].0, ax>
		@32 <mov [bx].4, cx>
		 call	rfsh_table
		jmp	chk_table


;Pri
tgl_active:	; ANeBu̐؂ւ
	_skip ne, dx, 0
		imul	bx, cx, 16
	 _skip e, <bptr EditTbl[bx].8>, dl
		mov	al, bptr EditTbl[bx].9
	  _repeat 4
		mov	bptr EditTbl[bx].9, dl
		add	bx, 16
		and	bx, 16*4-1
	  _loop
		and	al, BIT7
		xor	al, BIT7
		mov	bptr EditTbl[bx].9, al
		set	EditFlags, F_IC
		 call	rfsh_table
	__endif
		ret


;Exp
ins_item:	; ڑ}
	_skip ae, bp, 16
		imul	di, bp, 16
		add	di, ofs EditTbl[15]
		mov	si, di
		sub	si, 16
		add	ax, cx
	 _while b, ax, bp
		mov	cx, 16
		std
		rep movsb
		cld
		inc	ax
	 _rept
		@32 <mov ax, ExtArea[0]>
		@32 <mov [si].1, ax>
		@32 <mov [si].5, ax>
		mov	wptr[si].9, 0
		inc	EditItems
		set	EditFlags, F_CC
		 call	rfsh_table
		 call	chk_table
	_endif
		ret


;Exp
del_item:	; ڍ폜
		add	ax, cx
	_skip ae, ax, bp
		dec	EditItems
		imul	di, ax, 16
		add	di, ofs EditTbl
		mov	si, di
		add	si, 16
	 _while
		mov	cx, 16
		cld
		rep movsb
		inc	ax
	 _rept b, ax, EditItems
		mov	wptr[di].8, cx
		set	EditFlags, F_CC
		 call	rfsh_table
		 call	chk_table
	_endif
		ret


;Pri
enter_extend:	; gɓ
	_skip e, Lba, 0
	 _skip ne, dx, 0
		test	EditFlags, F_ER
	  _skip nz
		imul	bx, cx, 16
		mov	dx, EditTbl[bx].8
		 call	chk_extend
	   _skip ne
		 call	check_changed
	    _skip nz, <test EditFlags, F_IC+F_CC+F_NA>
		 call	read_ext_items
		mov	di, 80*12+0		;Show extend parameters
		mov	cx, 80*13		;
		 call	clr_block		;
		mov	di, 80*15+0		;
		mov	si, ofs MesExtend	;
		 call	put_stra		;
		@32 <mov ax, ExtArea[0]>	;
		mov	dx, 0f00h		;
		 call	put_num4		;
		 call	put_stra		;
		@32 <mov ax, ExtArea[4]>	;
		mov	dx, 0f00h		;
		 call	put_num4		;
		mov	di, 80*17+0		;
		mov	si, ofs HelpForm2	;
		 call	put_stra		;/
		 call	rfsh_table
		jmp	chk_table
	   __endif
	___endif
		jmp	beep


;Pri/Ext
save_table:	; e[u
		test	EditFlags, F_ER
		jnz	beep
		mov	si, ofs MesSave
		 call	req_enq
		ret ae
		mov	al, EditFlags
		and	al, F_CC+F_NA
	_if e, al, F_CC+F_NA
		mov	si, ofs MesNoAlign
		 call	req_enq
		ret ae
	_endif
		jmp	update_table


;Pri
chg_drive:	; ̃hCuɐ؂ւ
		 call	check_changed
		retn c
		inc	Drive
		mov	al, Drive
		and	al, 7fh
		cmp	al, Drives
	@if ae,<mov	Drive,80h>
		pop	ax
		jmp	edit_chg


;Pri
unformat:	; AtH[}bg
		test	EditFlags, F_ER+F_NA
		jnz	beep
		add	ax, cx
		imul	bp, ax, 16
		cje	<bptr EditTbl[bp].8>, 0, beep
		mov	si, ofs MesUnformat
		 call	req_enq
		ret ae
		FIL_STR  <ofs Pool>, 512, 0f6h	; fill with F6h
		@32 <mov cx, EditTbl[bp].0>
	_if e, Lba, 0
		clr	dh
	 _if e, cx, 0
		inc	dh
	 _endif
		xchg	ch, cl
		shl	cl, 6
		mov	dl, Drive
		mov	bx, ofs Pool
		mov	al, Sectors
	  _while
		push	ax
		inc	cl
		$DISK  3, 1
	  	pop	ax
	  _rept nz,<dec	al>
		ret
	_endif
		 call	cyl_size
		@32 <mul cx>
	_if z, <@32 <and cx,cx>>
		mov	al, Sectors
	_endif
		@32 <push 0>
		@32 <push ax>
		push	ds,ofs Pool, 1, 16
		clr	cx
		mov	cl, Sectors
		mov	dl, Drive
		mov	si, sp
	_repeat
		$DISK  43h,0
		@32 <inc wptr[si].8>
	_loop
		lea	sp, [si].16
		ret


;Pri
clr_ntsign:	; NTVOl`̃NA(ΏۃhCu)
		mov	si, ofs MesClrNT
		 call	req_enq
		ret ae
		 call	read_mbr	;MBRǂݍ
		FIL_STR <ofs MbrPartition[-16]>, 16, 0
		jmp	write_mbr	;MBR


;Pri
inst_mbm:	; u[g}l[W̃CXg[(2ndȍ~̂,擪l)
		cje	Drive,80h, beep
		mov	si, ofs MesInstall
		 call	req_enq
		ret ae

install_mbm_low:	; u[g}l[W̃CXg[ (Top stored)
		push	4
	_skip
install_mbm_high:	; u[g}l[W̃CXg[ (Bottom stored)
		push	63
	_endif
		mov	dl, Drive	;Get number of sectors/track
		$DISK  8		;/
		pop	ax
	_skip c
		and	cx, 63
	 _skip b, cl, N_MBM+3
		mov	ah, al
		add	ah, N_MBM-1
	  _if a, ah, cl
		sub	cl, N_MBM-1
	  _else
		mov	cl, al
	  _endif
		mov	Pool[0], cl
		clr	dx		;MBM{̂
		mov	dl, Drive	;
		mov	bx, ofs mbm_loader;
		$DISK  3, N_MBM		;/
	  _skip c
		clr	bp		;MBM{sumvZ
	   _repeat 512/2*N_MBM		;
		add	bp, [bx]	;
		incw	bx		;
	   _loop			;/
		 call	read_mbr	;MBRǂݍ
	    _skip c
		 call	chk_mbrbuff
		;MBR풓R[h
		MOV_MEM  <ofs mbr_code>,<ofs MbrBuffer>,<ofs mbr_end-mbr_code>
		;MBR풓f[^
		FIL_STR  <ofs MbrParms>,<type Parms>, 0
		mov	al, Pool[0]
		mov	MbrParms.MpMbmSect, al
		mov	MbrParms.MpMbmSum, bp
		mov	MbrParms.MpExp, -1
		 call	write_mbr
	____endif
		ret



enter_num:	; V_^VXeID̐l
		imul	di, cx, 80
		add	cx, EditOfsY
		imul	bp, cx, 16
		add	bp, ofs EditTbl
	_if e, dx, 0
		add	di, 80*4+3
		mov	dx, 0f003h
	_else
		imul	dx, 7
		add	dx, 80*4+20
		add	di, dx
		mov	dx, 0f006h
	_endif
	_skip b, al, '0'
	 _skip a, al, '9'
		mov	cx, ax
		$KEY  5
		clr	ax
		 call	input_str
	  _skip z
	   _if e, EditPosX, 0
		 call	get_valuehex
	   _else
		 call	get_value
	   _endif
		@32 <mov ax, cx>
	   _if e, EditPosX, 0
		xchg	al, [bp].8
		mul	bptr[bp].8
	    _if z, <test ax,ax>
		mov	[bp].9, al
		set	EditFlags, F_CC
	    _endif
		set	EditFlags, F_IC
	   _else
	    _if ae, <@32 <cmp ax, Cylinders>>
		@32 <mov ax, Cylinders>
		@32 <dec ax>
	    _endif
	    _if e, EditPosX, 1
		@32 <mov [bp].0, ax>
		@32 <cmp ax, [bp].4>
	     @if a,<@32 <mov [bp].4, ax>>
		mov	cx, 4d00h
		$KEY  5
	    _else
		@32 <mov [bp].4, ax>
		@32 <cmp ax, [bp].0>
	     @if b,<@32 <mov [bp].0, ax>>
	    _endif
		set	EditFlags, F_CC
	   _endif
		 call	chk_table
	  _endif
		 call	hide_csr
		 call	rfsh_table
	__endif
		ret



order_swap:
		$KEY  2
	_skip z,<test al, 11b>
		shl	si, 4
		shl	di, 4
		clr	bx			;Swap cylinder map table
	 _repeat 16				;
		mov al, bptr EditTbl[si][bx]	;
		xchg al, bptr EditTbl[di][bx]	;
		mov bptr EditTbl[si][bx], al	;
		inc	bx			;
	 _loop					;/
	 _skip ne, InExtend, 0
	  _repeat <type PartitionTbl>			;Pri:Swap partition table
		mov	al, bptr MbrPartition[si]	;
		xchg	al, bptr MbrPartition[di]	;
		mov	bptr MbrPartition[si], al	;
		inc	si				;
		inc	di				;
	  _loop						;/
	 _endif
		set	EditFlags, F_IC
		 call	chk_table
	_endif
		ret



;------------------------------------------------;


;Ext
read_ext_items:	; ǧƃe[u
		clr	ax				;Clear allocation parms
		FIL_STR <ofs EditTbl>,<16*16>,<>	;
		mov	EditItems, ax			;/
		imul	bp, EditPosY, type PartitionTbl
		add	bp, ofs MbrPartition
		@32 <mov ax, [bp].PartitionOfs>
		@32 <mov wptr Pool[0], ax>
		@32 <mov wptr Pool[4], ax>
		@32 <push ax>
		 call	cyl_num
		@32 <mov ExtArea[0], ax>
		@32 <pop ax>
		@32 <add ax, [bp].PartitionSize>
		@32 <dec ax>
		 call	cyl_num
		@32 <mov ExtArea[4], ax>
		mov	di, ofs EditTbl
exp_lp:		pusha
		 call read_pbr
		popa
		jc	exp_x
		cjne  MbrSign, 0aa55h, exp_x
		mov	si, ofs MbrPartition
	_repeat 4
		mov	dl, [si].SystemID
	 _if e, <call	chk_extend>
		@32 <mov ax, wptr Pool[0]>
		@32 <add ax, [si].PartitionOfs>
		@32 <mov [si].PartitionOfs, ax>
		@32 <mov wptr Pool[4], ax>
		mov	bp, si
		jmp	exp_lp
	 _endif
	 _skip e, dl, 0
		pusha
		 call	cyl_size
		@32 <mov bp, ax>
		mov	al, [si].SystemID
		mov	ah, 0
		mov	[di].8, ax
		@32 <mov ax, wptr Pool[4]>
		@32 <add ax, [si].PartitionOfs>
		@32 <mov bx, ax>
		 call	cyl_num
		@32 <mov [di].0, ax>
		@32 <mul bp>
		@32 <clr cx>
		mov	cl, Sectors
		@32 <add ax, cx>
		@32 <cmp ax, bx>
	  @ifn e,<set	EditFlags, F_NA>
		@32 <mov ax, bx>
		@32 <add ax,[si].PartitionSize>
		@32 <mov bx, ax>
		@32 <dec ax>
		 call	cyl_num
		@32 <mov [di].4, ax>
		@32 <inc ax>
		@32 <mul bp>
		@32 <cmp ax, bx>
	  @ifn e,<set	EditFlags, F_NA>
	 	popa
		add	di, 16
		inc	EditItems
		cjae	EditItems, 16, exp_x
	 _endif
		add	si, type PartitionTbl
	_loop
exp_x:		inc	InExtend
		jmp	clr_edit


;Pri
read_mbr_parm:	; hCup[^擾MBRǂݍ (CY:error, NC:succesful)
		 call	get_drvparm		;Get drive parameters
		ret c				;/
		 call	read_mbr		;Read MBR into MbrBuffer
		ret c				;
		FIL_STR <ofs EditTbl>,<16*16>,0	;Initialize allocation parms
		 call	chk_mbrbuff		;
		mov	bp, ofs EditTbl		;
		mov	cx, 4			;
		mov	EditItems, cx		;
	_repeat					;
		push	cx			;
		mov	al, [di].SystemID	;
		mov	ah, [di].BootID		;
	 _skip e, al, 0				;
		mov	[bp].8, ax		;
		@32 <mov ax,[di].PartitionOfs>	;
		 call	cyl_num			;
		@32 <mov [bp].0, ax>		;
		@32 <mov ax,[di].PartitionOfs>	;
		@32 <add ax,[di].PartitionSize>	;
		@32 <dec ax>			;
		 call	cyl_num			;
		@32 <mov [bp].4, ax>		;
	 _endif					;
		pop	cx			;
		add	bp, 16			;
		add	di, type PartitionTbl	;
	_loop					;/
		mov	InExtend, 0

clr_edit:
		clr	ax			;Clear cursor, flags
		mov	EditPosX, ax		;
		mov	EditPosY, ax		;
		mov	EditOfsY, ax		;
		mov	EditFlags, al		;/
		ret



check_changed:	; Check changed and update (NC:Canceled, CY:Passed)
		mov	al, EditFlags
		test	al, F_ER
		jnz	cc_no
		test	al, F_IC+F_CC
		jz	cc_no
		mov	si, ofs MesChange
		 call	req_enq
		ja	cc_can
		je	cc_no
		mov	al, EditFlags
		and	al, F_CC+F_NA
	_if e, al, F_CC+F_NA
		mov	si, ofs MesNoAlign
		 call	req_enq
		jae	cc_no
	_endif
		 call	update_table
cc_no:		stc
		ret
cc_can:		clc
		ret


;Pri/Ext
update_table:	; Partition tableXV
		clr	ax
		xchg	al, EditFlags
	_if e, ah, InExtend
		; MBR̍XV
		push	ax
		 call	build_table
		pop	ax
		mov	si, ofs Pool
	 _if nz, <test	al, F_CC>
		MOV_MEM <>,<>,<type PartitionTbl*4>
	 _else
	  _repeat 4
		mov	al, [si].SystemID
		mov	[di].SystemID, al
		mov	al, [si].BootID
		mov	[di].BootID, al
		add	si, type PartitionTbl
		add	di, type PartitionTbl
	  _loop
	 _endif
		jmp	write_mbr
	_endif

		; gXV
		mov	bp, ofs Pool
		mov	ax, EditItems
		mov	[bp].4, al
		 call	init_mbrbuff
		@32 <mov ax, ExtArea[0]>
		@32 <mov [bp].0, ax>
		mov	si, ofs EditTbl[-16]
	_while
	 _while ne, bptr[bp].4, 0		;Get next item
		dec	bptr[bp].4		;
		add	si, 16			;
	 _rept e, bptr[si].8, 0			;/
		pushf
	 _if ne
		mov	al, 05h			;CHS start/end
		clr	dh			;
		 call	put_extchs		;/
		@32 <mov ax, [si].0>		;LBA size
		@32 <sub ax, ExtArea[0]>	;
		@32 <mul bx>			;
	 	@32 <mov [di].PartitionOfs, ax>	;/
		@32 <mov ax, [si].4>		;LBA offset
		@32 <sub ax, [si].0>		;
		@32 <inc ax>			;
		@32 <mul bx>			;
	 	@32 <mov [di].PartitionSize, ax>;/
	 _endif
		push	si			;Write partition link
		 call	cyl_size		;
		@32 <mul wptr[bp].0>		;
		@32 <push 0>			;
		@32 <push ax>			;
		push	ds,ofs MbrBuffer, 1, 16	;
		mov	dl, Drive		;
		mov	si, sp			;
		$DISK  43h,0			;
		lea	sp, [si].16		;
		pop	si			;/
		popf
		ret e
		 call	init_mbrbuff
		@32 <mov ax, [si].0>		;Next partition
		@32 <mov [bp].0, ax>		;/
		mov	al, [si].8		;CHS start/end
		mov	dh, 1			;
		 call	put_extchs		;/
		@32 <clr ax>			;LBA offset
		mov	al, Sectors		;
		@32 <mov [di].PartitionOfs, ax>	;/
		@32 <mov ax, [si].4>		;LBA size
		@32 <sub ax, [si].0>		;
		@32 <inc ax>			;
		@32 <mul bx>			;
		@32 <clr dx>			;
		mov	dl, Sectors		;
		@32 <sub ax, dx>		;
		@32 <mov [di].PartitionSize, ax>;/
		add	di, type PartitionTbl
	_rept

put_extchs:	mov	[di].SystemID, al	;SystemID
		@32 <mov ax, [si].0>		;CHS start
		 call	clip_1023		;
		inc	al			;
		mov	[di].StartSectCyl, ax	;
		mov	[di].StartHead, dh	;/
		@32 <mov ax, [si].4>		;CHS end
		 call	clip_1023		;
		or	al, Sectors		;
		mov	[di].EndSectCyl, ax	;
		mov	al, Heads		;
		mov	[di].EndHead, al	;/
		 call	cyl_size		;ebx = cylinder size
		@32 <mov bx, ax>		;/
		ret



;Pri
scan_drives:	; ڑhCǔ
		mov	Drives, 0
		mov	al, 80h
	_while
		mov	dl, al
		push	ax
		$DISK  8
		pop	ax
	 _break c
	 _break be, dl, Drives
		inc	Drives
		inc	al
	_rept nae, Drives, 6
	_if e, Drives, 0
		pop	ax
	_endif
		ret



;Pri
chk_bound:	; ̃ACg`FbN
		 call	build_table
		mov	si, ofs Pool
	_repeat 4
	 _if ne, [si].SystemID, 0
		pusha
		mov	cx, 15
		inc	si
		inc	di
		repe cmpsb
		popa
	  _if ne
		set	EditFlags, F_NA
	 __endif
		add	si, type PartitionTbl
		add	di, type PartitionTbl
	_loop
		ret


;Pri/Ext
chk_table:	; ҏWe[u̗L`FbN
		mov	si, ofs EditTbl
		mov	cx, EditItems
	_while
	 _break cxz
		dec	cx
		push	si, cx
		@32 <mov ax, [si].0>		;Load area
		@32 <mov bx, [si].4>		;
		mov	dh, InExtend		;
		mov	dl, [si].8		;/
	 _if ne, dl, 0
		@32 <cmp ax, bx>		;Jn>I?
		ja	ct_err			;/
		@32 <cmp bx,Cylinders>		;TCYE?
		jae	ct_err			;/
	  _if e, dh, 0
	   _if z, <@32 <and ax,ax>>		;Pri:gCY0gp?
		 call	chk_extend		;
		je	ct_err			;
	   _endif				;/
	  _else
		 call	chk_extend		;Ext:g?
		je	ct_err			;/
		@32 <cmp ax, ExtArea[0]>	;Ext:TCYE?
		jb	ct_err			;
		@32 <cmp bx, ExtArea[4]>	;
		ja	ct_err			;/
	  _endif
	 _endif
	 _while
	  _break cxz
		dec	cx
		add	si, 16
	  _skip e, <bptr[si].8>, 0
	   _if ne, dl, 0			;d?
	    _if be, <@32 <cmp ax, [si].4>>	;
	     _if ae, <@32 <cmp bx, [si].0>>	;
		jmp	ct_err			;
	   ___endif				;/
	   _if ne, dh, 0			;Ext:㑱擪V_L?
		@32 <mov di, ExtArea[0]>	;
		@32 <cmp di, [si].0>		;
		je	ct_err			;
	   _endif				;/
	  _endif
	 _rept
		pop	cx, si
		add	si, 16
	_rept
		res	EditFlags, F_ER
	_skip
	ct_err:	pop	ax, ax
		set	EditFlags, F_ER
	_endif
		jmp	disp_stat


;Pri
build_table:	; e[u\z (ret:Pool[] = New table)
		mov	bp, ofs Pool
		FIL_STR bp,<type PartitionTbl*4>,0
		mov	si, ofs EditTbl
	_repeat 4
		push	cx
		mov	ax, [si].8
	 _skip e, al, 0
		mov	[bp].SystemID, al	;SystemID, BootID
		mov	[bp].BootID, ah		;/
		 call	cyl_size		; eax = sect/cyl
		@32 <push ax>			;
		@32 <mov dx, [si].4>		; ecx = p.size
		@32 <sub dx, [si].0>		; 
		@32 <inc dx>			; 
		@32 <mul dx>			; 
		@32 <mov cx, ax>		; /
		@32 <pop ax>
		@32 <mul wptr[si].0>		; eax = p.ofs
		clr	bx
	  _if e, <@32 <cmp wptr[si].0, 0>>	; adjust if cy# = 0
		mov	al, Sectors		;
		@32 <sub cx, ax>		;
		inc	bl			;
	  _endif				; /
		@32 <mov [bp].PartitionOfs, ax>
		@32 <mov [bp].PartitionSize, cx>
		mov	[bp].StartHead, bl
		mov	al, Heads
		mov	[bp].EndHead, al
		@32 <mov ax, [si].0>
		 call	clip_1023
		inc	al
		mov	[bp].StartSectCyl, ax
		@32 <mov ax, [si].4>
		 call	clip_1023
		or	al, Sectors
		mov	[bp].EndSectCyl, ax
	 _endif
		pop	cx
		add	bp, type PartitionTbl
		add	si, 16
	_loop
		mov	di, ofs MbrPartition
		ret



chk_mbrbuff:	; MBRL`FbN
	_if ne, MbrSign, 0aa55h
init_mbrbuff:	; MBRobt@NA
		FIL_STR <ofs MbrBuffer>,<size MbrBuffer-2>,0
		mov	wptr[di], 0aa55h
	_endif
		mov	di, ofs MbrPartition
		ret


;------------------------------------------------;

disp_stat:	; ҏWe[u̗LEAgpEgpTCY̕\
	_if ne, InExtend, 0
		test	EditFlags, F_ER
		jz	clr_message
		mov	si, ofs MesDup
		jmp	put_message
	_endif
	_if nz, <test EditFlags, F_ER>		;Error
		mov	ah, 07h			;
		mov	si, ofs StrSpace[0]	;
		mov	di, 80* 9+60		;
		 call	put_str			;
		mov	si, ofs StrSpace[0]	;
		mov	di, 80* 10+60		;
		 call	put_str			;
		mov	si, ofs MesDup		;
		jmp	put_message		;/
	_endif
		clr	bp			; eax = used in MB
		@32 <clr dx>			;
	_repeat 4				;
	 _if ne, <bptr EditTbl[bp].8>, 0	;
		@32 <add dx, EditTbl[bp].4>	;
		@32 <sub dx, EditTbl[bp].0>	;
		inc	dx			;
	 _endif					;
		add	bp, 16			;
	_loop					;
		 call	cyl_size		;
		@32 <mul dx>			; /
		@32 <push ax>
  		mov	di, 80* 9+60		; Used
		 call	put_size2		; /
		 call	cyl_size		; eax = total in MB
		@32 <mul Cylinders>		;
		@32 <pop dx>			;
		@32 <sub ax,dx>			; eax = total - used = free
  		mov	di, 80* 10+60		; Free
		 call	put_size2		; /
		jmp	clr_message



disp_parm:	; hCu̕\
		mov	di, 80*1+57	; disk #
		mov	ah, Drive
		and	ah, 7fh
		clr	al
	_while
		cmp	al, ah
		push	ax
		mov	ah, 07h
	 @if e,<mov	ah, 0f0h>
		add	al, '0'
		 call	put_chr
		pop	ax
		inc	al
		inc	di
	_rept nae, al, Drives
		mov	ChrAttr, 0fh
		cmp	Lba, 0		; LBA status
		mov	si, ofs LbaDis
	@ifn e,<mov	si, ofs LbaEna>
		mov	di, 80* 2+55
		 call	put_stra
		mov	al, Sectors	; secors
		mov	di, 80* 4+61
		mov	dx, 0f05h
		 call	put_num1
		clr	ax		; heads
		mov	al, Heads
		inc	ax
		mov	di, 80* 5+61
		mov	dx, 0f05h
		 call	put_num2
		@32 <mov ax, Cylinders>	; cyl.
		mov	di, 80* 6+60
		mov	dx, 0f06h
		 call	put_num4
		 call	cyl_size	; total
		@32 <mul Cylinders>
		mov	di, 80* 7+60
		jmp	put_size2



rfsh_table:	; ҏW(SystemID, Start, End, Size, BootID)̕\
		clr	ax, dx
	_repeat 7
		push	cx
		clr	dx
	 _repeat 5
		pusha
		mov	cl, 0fh
	  _if e, ax, EditPosY
	   _if e, dx, EditPosX
		rol	cl, 4
	  __endif
		 call	rfsh_item
		popa
		inc	dx
	 _loop
		pop	cx
		inc	ax
	_loop
		ret


rfsh_item:					; 捀ڂ̕\
		imul	di, ax, 80		; Call: ax=Y, dx=X, cl=color
		add	di, 80*4
		add	ax, EditOfsY
		imul	bp, ax, 16
		mov	ch, bptr EditTbl[bp].8
	_if ae, ax, EditItems
		clr	ch
	_endif

	_if e, dl, 0	; Partition type
		add	di, 3
		push	di, cx
		FIL_STR <ofs Pool>, 23, <' '>
		pop	cx
		mov	bptr[di], 0
	 _skip e, ch, 0
		mov	di, ofs Pool
		mov	al, ch
		 call	cv_hex
		stosw
		mov	al, ':'
		stosb
		mov	dl, ch
		 call	chk_extend
	  @if e,<and	cl, 0bbh>
		mov	ah, ch
		 call	get_osname
	  _while
		movsb
	  _rept ne, bptr[si], 0
	 _endif
		pop	di
		mov	ah, cl
		mov	si, ofs Pool
		jmp	put_str
	_endif

	_if be, dl, 2	; Start/End cylinder
		@32 <mov ax, EditTbl[bp].0>
	 _if e
		@32 <mov ax, EditTbl[bp].4>
	 	add	di, 7
	 _endif
		add	di, 27
	 _if e, ch, 0
		mov	si, ofs StrSpace[2]
		mov	ah, cl
		jmp	put_str
	 _endif
		mov	dh, cl
		mov	dl, 6
		jmp	put_num4
	_endif

	_if e, dl, 3	; Size
		add	di, 41
	 _if e, ch, 0
		mov	si, ofs StrSpace[1]
		mov	ah, cl
		jmp	put_str
	 _endif
		 call	cyl_size
		@32 <mov dx, ax>
		@32 <mov ax, EditTbl[bp].4>
		@32 <sub ax, EditTbl[bp].0>
		@32 <inc ax>
		@32 <mul dx>
		jmp	put_size2
	_endif
			; Item
		inc	di
	_if ae, ax, EditItems
		mov	al, ' '
	_else
		tst	InExtend
	 @ifn z,<add	al, 4>
		add	al, '1'
		cmp	al, '9'
	 @if a, <add	al, 7>
	_endif
		mov	ah,0ah
		test	bptr EditTbl[bp].9, BIT7
	@if z,<mov	ah,07h>
		jmp	put_chr




req_enq:	; mF
		 call	put_message
		 call	move_csr
		 call	show_csr
	_while
		$KEY	0
		CAPS	al
	 _break e, al,'Y'
	 _break e, al,'N'
	_rept ne, al,ESC
		mov	ah, 2
		cmp	al, 'Y'
	@if e, <clr	ah>
		cmp	al, 'N'
	@if e, <dec	ah>
		push	ax
		 call	clr_message
		 call	hide_csr
		pop	ax
		cmp	ah, 1
		ret


get_osname:				; Call:
		mov	si, ofs OsTable	;  ah = OS code
	_while				; Ret:
		lodsb			;  si -> String
		ret e, ah, al
	 _while
		lodsb
	 _rept ne, al,0
	_rept ne, bptr[si], 0
		inc	si
		ret


clr_message:	; bZ[W̏(e[uGfB^)
		mov	di, 80*12+0
		mov	cx, 80*2
		jmp	clr_block



;-------------------------------------------------------;
; ėp񑀍


scan_cmd:	; L[R}h bx -> command table
	_if ne, al,0
		CAPS	al
		clr	ah
	_endif
	_while
	 _if e, ah, 0
		cmp	al, [bx].0
	 _else
		cmp	ah, [bx].1
	 _endif
	 _if e
		mov	bx, [bx].2
		ret
	 _endif
		add	bx, 4
		mov	dl, -1
	_rept ne, [bx], dl
		or	dl, dl
		ret


clr_screen:	; ʃNA
		clr	bl		;Disable brink
		$VIDEO  10h, 3		;/
		clr	di		;Clear screen
		mov	cx, 80*25	;
		 call	clr_block	;/
		 call	move_csr
show_csr:	; J[\\
		mov	cx, 0607h
	_skip
hide_csr:	; J[\
		mov	cx, 2020h
	_endif
		clr	bh
		$VIDEO  1
		ret

move_csr:	; J[\ړ(di=screen addr.)
		pusha
		mov	ax, di
		mov	dl, 80
		div	dl
		mov	dl, ah
		mov	dh, al
		clr	bh
		$VIDEO	2
		popa
		ret



put_strf:	; \(ŒF)
		mov	ah, ChrAttr
		jmp	put_str
put_message:	; bZ[W̕\
		mov	di, 80* 12+0
put_stra:	; \(wF)
		mov	ah, [si]
		inc	si
put_str:	; \
		push	dx
	_while
		lodsb
	 _break e, al, 0
		 call	put_chr
		mov	dl, al
	_rept
		pop	dx
		ret

		; ̕\ (AH = attr, AL = char)
put_chrf:	mov	ah, ChrAttr
put_chr:	push	es
		movseg	es, 0b800h
		shl	di, 1
	 _case e, al,CR
		push	ax
		mov	ax, di
		mov	dl, 160
		div	dl
		mov	ah, 160
		mul	ah
		add	ax, 160
		mov	di, ax
		pop	ax
	 _case e, al,BS
		decw	di
		mov	al,' '
		mov	es:[di], ax
	 _case e, al,TAB
		mov	al, dl
	  _while
		stosw
		test	di, 15
	  _rept nz
	 _default
		stosw
	 _endc
		shr	di, 1
		pop	es
		ret


put_num1:	clr	ah		; l̕\
put_num2:	@32 <and ax,-1>,0	; Call:
put_num4:	push	cx		;  eax = number, di = position
		clr	bx		;  dh = attr, dl = number of digits
	_while
		inc	bx
		mov	cx, 10
		 call	div_eax
		add	cl, '0'
		push	cx
		@32 <and ax, ax>
	_rept nz
	_while b, bl,dl
		inc	bx
		push	'  '
	_rept
	_repeat bx
		pop	ax
		mov	ah, dh
		 call	put_chr
	_loop
		pop	cx
		ret


clr_block:	; ubN (di=ʒu, cx=)
		push	di
	_repeat
		mov	ax, 0720h
		 call	put_chr
	_loop
		pop	di
		ret


beep:		; r[v
		clr	bh
		$VIDEO  0eh, BEL
		ret


		; Poolɕ (dh=attr, dl=max char)
input_strn:	mov	dx, 0f00h+type NameItem-1
input_str:	push	ax, di, dx, di
		 call	move_csr
	_while
		mov	ah, dh
		mov	al, ' '
		 call	put_chr
		dec	dl
	_rept nz
		pop	di
		 call	move_csr
		 call	show_csr
		FIL_STR <ofs Pool>, 50, 0
		pop	dx, di, ax
		clr	bx
	_skip e, ax, 0
		mov	si, ax
	 _while
		lodsb
	  _break e, al, 0
		mov	Pool[bx], al
		inc	bx
		mov	ah, dh
		 call	put_chr
		 call	move_csr
	 _rept b, bl, dl
	_endif
	_while
		$KEY  0
		clr	ah
	 _break e, al, CR
	 _break e, al, ESC
	 _if e, al, BS
	  _skip e, bl, ah
		dec	bx
		mov	Pool[bx], ah
		mov	ah, dh
		 call	put_chr
		 call	move_csr
	  _endif
		clr	ax
	 _endif
	 _cont b, al, ' '
	 _cont ae, bl, dl
		mov	wptr Pool[bx], ax
		inc	bx
		mov	ah, dh
		 call	put_chr
		 call	move_csr
	_rept
		cmp	al, ESC
		ret


get_value:	; 10il擾 Call:Pool = decimal string, Ret:ECX=value
		mov	si, ofs Pool
		@32 <clr cx>
	_while
		lodsb
	 _break e, al, 0
		sub	al, '0'
	 _break b
		cmp	al, 10
		cmc
	 _break b
		@32 <imul cx,10>
		@32 <and ax, 255>,0
		@32 <add cx, ax>
	_rept
		ret


get_valuehex:	; 16il擾 Call:Pool = decimal string, Ret:CX=value
		mov	si, ofs Pool
		clr	ax,cx
	_while
		lodsb
		CAPS	al
	 _break a, al, 'F'
		sub	al, '0'
	 _break b
		cmp	al, 10
	 @if ae,<sub	al, 7>
		shl	cx, 4
		add	cx, ax
	_rept
		ret


cv_hex:		; 16iϊ AL -> AX
		ror	ax, 4
		shr	ah, 4
		and	al, 0fh
		cmp	al, 10
	   @if ae, <add	al, 7>
		cmp	ah, 10
	   @if ae, <add	ah, 7>
		add	ax, '00'
		ret


cyl_num:	; Get cylinder # form sector#
		mov	cl, Sectors
		 call	div_eax1
		mov	cl, Heads
		inc	cx
		jmp	div_eax


clip_1023:	; Get CHS value from cylinder #
	  	@32 <cmp ax, 1023>,0
	  _if a
		mov	ax, 1023
	  _endif
		xchg	ah, al
		shl	al, 6
		ret









;----------------------------------------------------------------------------;
; u[g}l[WMBR풓

mbr_code:	; (BIOSɂ0000:7c00hփ[h)
	_skip
MbmSign		db	'MBM R',VER,': Loading...',0
MbmLost		db	'MBM has been broken.',0
	_endif
		clr	ax			; WX^̏
		mov	ds, ax
		mov	es, ax
		mov	ss, ax
		mov	sp, ofs NameBuffer
err_retry:
		cld
		mov	si, 7c00h+ ofs MbmSign-mbr_code ; NbZ[W
	_while
		lodsb
	 _break e, al, 0
		clr	bh
		$VIDEO  0eh
	_rept

		mov	bx, ofs mbm_loader	;{̂̃[h
		clr	cx			;Cy:0
		mov	cl, MbrParms.MpMbmSect	;Sc
		mov	dx, 0080h		;Hd:0, Dr:0
		$DISK  2, N_MBM			;Read
		jc	err_retry
		push	bx		; Save mbm offset

		clr	ax		; {SUMmF
	_repeat 512/2*N_MBM		;
		add	ax, [bx]	;
		incw	bx		;
	_loop				;
	_skip e, ax, MbrParms.MpMbmSum	;/
		mov	si, 7c00h+ ofs MbmLost-mbr_code ; sbZ[W
	 _while
		lodsb
		tst	al
		jz	$
		clr	bh
		$VIDEO  0eh
	 _rept
	_endif

		mov	si, ofs MbrParms	;p[^ړ
		mov	di, ofs MainParms	;
		mov	cx, type Parms		;
		rep movsb			;
		mov	ax, MbrParms.MpDef	;
		mov	MenuValue, ax		;/

		ret				; MBM{̂

		even

mbr_end:	; MBR풓I


;----------------------------------------------------------------------------;
; u[gj[ƃGfB^̕


		; BOOT MANAGER

MbmTitle	db	1fh,' Multiple Boot Manager R',VER,' 						',0
		db	'F1:Help ',0
TimerStr	db	0e8h,' START ',0
		db	'',0
		db	0ch, '* ',0

HelpStr		db	'Enter^M',0,':Start',0
		db	'1-9^EX',0,':Select item',0
		db	'^SD',0,':Expand/Shrink',0
		db	'SP',0,':Set default drv',0
		db	'F1H',0,':Hide help',0
		db	'F2R',0,':Rename menu item',0
		db	'F3T',0,':Set timer',0
		db	'F4V',0,':Verbose...',0
		db	'F5A',0,':Activate',0
		db	'F6B',0,':Boot mode',0
		db	'F7M',0,':Mask/Unmask',0
		db	'F8K',0,':Auto mask group',0
		db	'F10',0,':Edit partition table',0
		db	'Del',0,':Retry boot sequence',0
		db	'End',0,':Shut down',0
		db	0

DiskErr		db	0ch, 'Disk error',0
InvalidPbr	db	0ch, 'Invalid boot record',0
ChgTimer	db	07h, 'Auto start time[sec]=',0

StrDrv		db	' :',0
StrModes	db	'[    ]',0,'   ',0
StrSpace	db	'        ',0

FdBoot		db	OFF	; Running at FD boot mode


		; EDITOR

MesSave		db	0eh,'Current setting will be written.',CR
		db	'Do you wish to continue? <y/n>=',0
MesChange	db	0eh,'The parameters have been modified.',CR
		db	'Do you wish to write changes? <y/n/esc>=',0
MesNoAlign	db	0ch,'Any partition will be re-aligned to cylinder boundary.',CR
		db	'It may be destroyed. Do you wish to continue? <y/n>=',0
MesUnformat	db	0ch,'This partition will be unformatted.',CR
		db	'Are you sure? <y/n>=',0
MesInstall	db	0eh,'MBM will be installed into this drive.',CR
		db	'Are you sure? <y/n>=',0
MesClrNT	db	0eh,'Clear NT signature. <y/n>=',0
MesDup		db	0ch,'Error: Partitioning rule violation.',0
MesExtend	db	0bh,'Inside extend partition from ',0,0bh,' to ',0

MesNoDisk	db	BEL,'No disk.',0


EditForm	db	07h
		db     '								ͻ',CR
		db   ' MBM R',VER,' 		   Start End   Size   Drive: 	    ',CR
		db     ' Partition Table Editor   Cyl.  Cyl.  	 LBA: 		    ',CR
		db     '						 		 ',CR
		db     '  			   	  	 	 Sectors: 	    ',CR
		db     '  			   	  	 	 Heads: 	    ',CR
		db     '  			   	  	 	 Cylinders: 	    ',CR
		db     '  			   	  	 	 Capacity: 	    ',CR
		db     '  			   	  	 	 		 ',CR
		db     '  			   	  	 	 Used: 	    ',CR
		db     '  			   	  	 	 Free: 	    ',CR
		db     '								ͼ',CR
		db	CR,CR,CR
		db     ': Move cursor',CR
		db     '0-9PUPD: Change value',CR
		db     'Sft+: Change order',CR
		db     'F5: Unformat partition',CR
		db     'F8: Update changes',CR
		db     'F9: Install boot manager',CR
		db     'F10: Next drive',CR
		db     'SP: Activate/Deactivate',CR
		db     'Enter: Enter extend partition',CR
		db     'Esc: Exit editor',CR
		db	0
HelpForm2	db	07h
		db     ': Move cursor',CR
		db     '0-9PUPD: Change value',CR
		db     'Sft+: Change order',CR
		db     'F8: Update changes',CR
		db     'Ins: Insert item',CR
		db     'Del: Delete item',CR
		db     'Esc: Exit extend partition',CR
		db	0

LbaEna		db	0fh,'Enabled ',0
LbaDis		db	06h,'Disabled',0

StrFdd		db	'Diskette',0
OsTable		db	01h,'DOS FAT12',0
		db	04h,'DOS FAT16(<32M)',0
		db	05h,'DOS Extend',0
		db	06h,'DOS FAT16(32M-2G)',0
		db	07h,'HPFS/NTFS',0
		db	0bh,'DOS FAT32',0
		db	0ch,'DOS FAT32X',0
		db	0eh,'DOS FAT16X',0
		db	0fh,'DOS ExtendX',0
		db	11h,'DOS FAT12',0
		db	13h,'B-right/V',0
		db	14h,'DOS FAT16(<32M)',0
		db	15h,'DOS Extend',0
		db	16h,'DOS FAT16(32M-2G)',0
		db	17h,'HPFS/NTFS',0
		db	1bh,'DOS FAT32',0
		db	1ch,'DOS FAT32X',0
		db	1eh,'DOS FAT16X',0
		db	1fh,'DOS ExtendX',0
		db	39h,'Plan 9',0
		db	71h,'EOTA',0
		db	81h,'Minix/Linux extfs',0
		db	82h,'Solaris/Linux swap',0
		db	83h,'Linux ext2fs',0
		db	85h,'Linux Extend',0
		db	0a0h,'Suspend',0
		db	0a4h,'FreeBSD',0
		db	0a5h,'FreeBSD',0
		db	0a6h,'OpenBSD',0
		db	0a9h,'NetBSD',0
		db	0c1h,'DRDOS',0
		db	0c4h,'DRDOS',0
		db	0c6h,'DRDOS(32M-2G)',0
		db	0ebh,'BeOS',0
		db	0,   'Unknown',0

;

;------------ u[g}l[W{̏I
end_of_mbm	label	near
; 2801..2A00: N_MBM = 17
; 2A01..2C00: N_MBM = 18
; 2C01..2E00: N_MBM = 19
; 2E01..3000: N_MBM = 20

		even
		db	512 dup(0)	; PADDING

;-----------------------------------------------;
; MBMCXg[(DOS/FD)


FdMode		db	0

StrFdUsage	db	CR,LF
		db	'***** MBM R',VER,' diskette image mode *****',CR,LF
		db	' 1. Boot menu',CR,LF
		db	' 2. Edit partition table.',CR,LF
		db	' 3. Install boot manager',CR,LF
		db	' 4. Same as 3, but last sector in the track.',CR,LF
		db	' 5. Restart.',CR,LF
		db	'Choose operation (1-5)=',0

StrDrvNum	db	'Drive number to install (0-3)=',0


StrDosOpen	db	'MBM - Multiple Boot Manager  R',VER, '  (C)ChaN, 2008.', CR,LF
		db	CR,LF,0

StrDosUsage	db	'Usage: mbm <command>',CR,LF
		db	CR,LF
		db	'<command>:',CR,LF
		db	' install [<drv#>] - Install boot manager.',CR,LF
		db	' install_h [<drv#>] - Same as install, but last sector in the track.',CR,LF
		db	' edit - Edit partition table.',CR,LF
		db	' setmenu - Install extend menu.',CR,LF
		db	' clrmenu - Remove extend menu.',CR,LF
		db	0

MesNoInst	db	BEL,'MBM R',VER,' is not installed.',0
MesNoLba	db	BEL,'No LBA support.',0
ReqFile		db	'Menu file (*.mbm) = ',0
NoFile		db	'No file.',CR,LF,0
ReqCyl		db	'Cylinder# to install(0-',0,') = ',0
MesInvalid	db	BEL,'Invalid cylinder number.',0
MesCyl		db	LF,BEL,'WARNING: This cylinder is already in use!',CR,LF,0

MesInst		db	'The boot manager will be installed into drive #',0
MesCont		db	CR,LF,'Do you wish to continue? <Y/n>=',0
MesWrite	db	'Installing....',0
MesOK		db	'done.',CR,LF,0
MesLarge	db	'too large menu. Aborted.',CR,LF,0
StrIOerr	db	'I/O error!'
CrLf		db	CR,LF,0

		even



;----------------------------------------------------------------------------;
; u[g}l[WEGfB^[NGA

MenuItems	dw	?	; Number of menu items
MenuValue	dw	?	; Current menu value
Expand		dw	?	; Expanded partition (partition, drive)
BootMode	db	?,?,?	; Current boot mode, mask mode,
ErrRetry	db	?	; Load and go retry counter

EditPosX	dw	?	; Position (parm)
EditPosY	dw	?	; Position (item)
EditItems	dw	?	; Number or items
EditOfsY	dw	?	; Display offset
ExtArea		dw	?,?,?,?	; Current extend partition area
EditTbl		dw	16*16 dup(?) ; Start.Cyl, End.Cyl, ID, Boot
EditFlags	db	?	; Parameter changed flags (b0:Act/ID/Order changed, b1:Cylinder changed, b2:Alloc error, b3:Not aligned)

ChrAttr		db	?	; Fixed character attribute

InExtend	db	?	; In extend partition editor
Drives		db	?	; Number of disks
Drive		db	?	; R/W drive# (00h, 80h..
Lba		db	?	; LBA mode. 0:Disabled, 1:Enabled
Sectors		db	?	; Number of sectors
Heads		db	?	; Maximum head#
Cylinders	dw	?,?	; Number of cylinders

Installed	db	4 dup(?); Is MBM installed?
MainParms	Parms	<>

MenuTable	db	21 dup(?,?,?,?) ; j[ڂ̋AhX (Par,Drv,ID,Act)

PbrList		PartitionTbl	<>	; 16x(16+20)

Argc		db	?
Argv		dw	?
Hand		dw	?




		org	6c00h		; j[ږe[u
NameBuffer	NameItem	<>	; 0-0
		NameItem	<>	; 0-1
		NameItem	<>	; 0-2
		NameItem	<>	; 0-3
		NameItem	<>	;
		NameItem	<>	;
		NameItem	<>	;
		NameItem	<>	;
		NameItem	<>	;
		NameItem	<>	;
		NameItem	<>	;
		NameItem	<>	;
		NameItem	<>	;
		NameItem	<>	;
		NameItem	<>	;
		NameItem	<>	;
		NameItem	<>	; 4-0 FDD
		db	512 dup(?)	; Disk#0 logical

		db	512 dup(?)	; Disk#1 primary
		db	512 dup(?)	; Disk#1 logical
		db	512 dup(?)	; Disk#2 primary
		db	512 dup(?)	; Disk#2 logical
		db	512 dup(?)	; Disk#3 primary
		db	512 dup(?)	; Disk#3 logical


		org	7c00h
MbrBuffer	db	512 dup(?)	; u[gR[hǂݏobt@

		org	7c00h + (ofs mbr_end-mbr_code)	; ep[^
 MbrParms	Parms	<>

		org	7c00h + 1beh	; e[uiMBR̂݁j
 MbrPartition	PartitionTbl	<>
		PartitionTbl	<>
		PartitionTbl	<>
		PartitionTbl	<>
 MbrSign	dw	?		; VOl`iAA55hj

Pool		db	100 dup(?)	; ̓obt@


		$end	start

